@@ -28,6 +28,7 @@
 #include <taler/taler_templating_lib.h>
 #include <taler/taler_exchange_service.h>
 #include "taler-merchant-httpd_exchanges.h"
+#include "taler-merchant-httpd_helper.h"
 #include "taler-merchant-httpd_get-orders-ID.h"
 #include "taler-merchant-httpd_mhd.h"
 #include "taler-merchant-httpd_qr.h"
@@ -118,6 +119,22 @@ struct GetOrderData
    */
   json_t *contract_terms;
 
+  /**
+   * Merchant base URL from @e contract_terms.
+   */
+  const char *merchant_base_url;
+
+  /**
+   * Public reorder URL from @e contract_terms.
+   * Could be NULL if contract does not have one.
+   */
+  const char *public_reorder_url;
+
+  /**
+   * Total amount in contract.
+   */
+  struct TALER_Amount contract_total;
+
   /**
    * Total refunds granted for this payment. Only initialized
    * if @e refunded is set to true.
@@ -144,6 +161,16 @@ struct GetOrderData
    */
   enum GNUNET_GenericReturnValue suspended;
 
+  /**
+   * Set to true if the client passed 'h_contract'.
+   */
+  bool h_contract_provided;
+
+  /**
+   * Set to true if the client passed a 'claim' token.
+   */
+  bool claim_token_provided;
+
   /**
    * Set to true if we are dealing with a claimed order
    * (and thus @e h_contract_terms is set, otherwise certain
@@ -169,6 +196,16 @@ struct GetOrderData
    */
   bool generate_html;
 
+  /**
+   * Did we parse the contract terms?
+   */
+  bool contract_parsed;
+
+  /**
+   * Set to true if the refunds found in the DB have
+   * a different currency then the main contract.
+   */
+  bool bad_refund_currency_in_db;
 };
 
 
@@ -305,6 +342,9 @@ suspend_god (struct GetOrderData *god)
     json_decref (god->contract_terms);
     god->fulfillment_url = NULL;
     god->contract_terms = NULL;
+    god->contract_parsed = false;
+    god->merchant_base_url = NULL;
+    god->public_reorder_url = NULL;
   }
   GNUNET_assert (! god->suspended);
   god->suspended = GNUNET_YES;
@@ -377,62 +417,26 @@ TMH_make_order_status_url (struct MHD_Connection *con,
                            struct TALER_ClaimTokenP *claim_token,
                            struct TALER_PrivateContractHashP *h_contract)
 {
-  const char *host;
-  const char *forwarded_host;
-  const char *uri_path;
-  struct GNUNET_Buffer buf = { 0 };
+  struct GNUNET_Buffer buf;
   /* Number of query parameters written so far */
   unsigned int num_qp = 0;
 
-  host = MHD_lookup_connection_value (con,
-                                      MHD_HEADER_KIND,
-                                      MHD_HTTP_HEADER_HOST);
-  forwarded_host = MHD_lookup_connection_value (con,
-                                                MHD_HEADER_KIND,
-                                                "X-Forwarded-Host");
-  uri_path = MHD_lookup_connection_value (con,
-                                          MHD_HEADER_KIND,
-                                          "X-Forwarded-Prefix");
-  if (NULL != forwarded_host)
-    host = forwarded_host;
-  if (NULL == host)
+  GNUNET_assert (NULL != instance_id);
+  GNUNET_assert (NULL != order_id);
+  if (GNUNET_OK !=
+      TMH_base_url_by_connection (con,
+                                  instance_id,
+                                  &buf))
   {
     GNUNET_break (0);
     return NULL;
   }
-  if (NULL != strchr (host, '/'))
-  {
-    GNUNET_break_op (0);
-    return NULL;
-  }
-  GNUNET_assert (NULL != instance_id);
-  GNUNET_assert (NULL != order_id);
-
-  if (GNUNET_NO == TALER_mhd_is_https (con))
-    GNUNET_buffer_write_str (&buf,
-                             "http://");
-  else
-    GNUNET_buffer_write_str (&buf,
-                             "https://");
-  GNUNET_buffer_write_str (&buf,
-                           host);
-  if (NULL != uri_path)
-    GNUNET_buffer_write_path (&buf,
-                              uri_path);
-  if (0 != strcmp ("default",
-                   instance_id))
-  {
-    GNUNET_buffer_write_path (&buf,
-                              "instances");
-    GNUNET_buffer_write_path (&buf,
-                              instance_id);
-  }
   GNUNET_buffer_write_path (&buf,
                             "/orders");
   GNUNET_buffer_write_path (&buf,
                             order_id);
-  if ((NULL != claim_token) &&
-      (GNUNET_NO == GNUNET_is_zero (claim_token)))
+  if ( (NULL != claim_token) &&
+       (! GNUNET_is_zero (claim_token)) )
   {
     /* 'token=' for human readability */
     GNUNET_buffer_write_str (&buf,
@@ -480,60 +484,27 @@ TMH_make_taler_pay_uri (struct MHD_Connection *con,
                         const char *instance_id,
                         struct TALER_ClaimTokenP *claim_token)
 {
-  const char *host;
-  const char *forwarded_host;
-  const char *uri_path;
-  struct GNUNET_Buffer buf = { 0 };
+  struct GNUNET_Buffer buf;
 
-  host = MHD_lookup_connection_value (con,
-                                      MHD_HEADER_KIND,
-                                      MHD_HTTP_HEADER_HOST);
-  forwarded_host = MHD_lookup_connection_value (con,
-                                                MHD_HEADER_KIND,
-                                                "X-Forwarded-Host");
-  uri_path = MHD_lookup_connection_value (con,
-                                          MHD_HEADER_KIND,
-                                          "X-Forwarded-Prefix");
-  if (NULL != forwarded_host)
-    host = forwarded_host;
-  if (NULL == host)
+  GNUNET_assert (NULL != instance_id);
+  GNUNET_assert (NULL != order_id);
+  if (GNUNET_OK !=
+      TMH_taler_uri_by_connection (con,
+                                   "pay",
+                                   instance_id,
+                                   &buf))
   {
     GNUNET_break (0);
     return NULL;
   }
-  if (NULL != strchr (host, '/'))
-  {
-    GNUNET_break_op (0);
-    return NULL;
-  }
-  GNUNET_assert (NULL != instance_id);
-  GNUNET_assert (NULL != order_id);
-  GNUNET_buffer_write_str (&buf,
-                           "taler");
-  if (GNUNET_NO == TALER_mhd_is_https (con))
-    GNUNET_buffer_write_str (&buf,
-                             "+http");
-  GNUNET_buffer_write_str (&buf,
-                           "://pay/");
-  GNUNET_buffer_write_str (&buf,
-                           host);
-  if (NULL != uri_path)
-    GNUNET_buffer_write_path (&buf,
-                              uri_path);
-  if (0 != strcmp ("default",
-                   instance_id))
-  {
-    GNUNET_buffer_write_path (&buf,
-                              "instances");
-    GNUNET_buffer_write_path (&buf,
-                              instance_id);
-  }
   GNUNET_buffer_write_path (&buf,
                             order_id);
   GNUNET_buffer_write_path (&buf,
-                            (session_id == NULL) ? "" : session_id);
-  if ((NULL != claim_token) &&
-      (GNUNET_NO == GNUNET_is_zero (claim_token)))
+                            (NULL == session_id)
+                            ? ""
+                            : session_id);
+  if ( (NULL != claim_token) &&
+       (! GNUNET_is_zero (claim_token)))
   {
     /* Just 'c=' because this goes into QR
        codes, so this is more compact. */
@@ -770,6 +741,16 @@ process_refunds_cb (void *cls,
               TALER_B2S (coin_pub),
               reason);
   god->refund_pending |= pending;
+  if ( (GNUNET_OK !=
+        TALER_amount_cmp_currency (&god->refund_taken,
+                                   refund_amount)) ||
+       (GNUNET_OK !=
+        TALER_amount_cmp_currency (&god->refund_amount,
+                                   refund_amount)) )
+  {
+    god->bad_refund_currency_in_db = true;
+    return;
+  }
   if (! pending)
   {
     GNUNET_assert (0 <=
@@ -824,10 +805,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   enum GNUNET_DB_QueryStatus qs;
   bool contract_match = false;
   bool token_match = false;
-  bool h_contract_provided = false;
-  bool claim_token_provided = false;
   bool contract_available = false;
-  const char *merchant_base_url;
 
   (void) rh;
   if (NULL == god)
@@ -840,53 +818,15 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
     god->order_id = order_id;
     god->generate_html = TMH_MHD_test_html_desired (connection);
 
-
     /* first-time initialization / sanity checks */
-    {
-      const char *cts;
-
-      cts = MHD_lookup_connection_value (connection,
-                                         MHD_GET_ARGUMENT_KIND,
-                                         "h_contract");
-      if ( (NULL != cts) &&
-           (GNUNET_OK !=
-            GNUNET_CRYPTO_hash_from_string (cts,
-                                            &god->h_contract_terms.hash)) )
-      {
-        /* cts has wrong encoding */
-        GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_BAD_REQUEST,
-                                           TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                           "h_contract");
-      }
-      if (NULL != cts)
-        h_contract_provided = true;
-    }
-
-    {
-      const char *ct;
-
-      ct = MHD_lookup_connection_value (connection,
-                                        MHD_GET_ARGUMENT_KIND,
-                                        "token");
-      if ( (NULL != ct) &&
-           (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (ct,
-                                           strlen (ct),
-                                           &god->claim_token,
-                                           sizeof (god->claim_token))) )
-      {
-        /* ct has wrong encoding */
-        GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_BAD_REQUEST,
-                                           TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                           "token");
-      }
-      if (NULL != ct)
-        claim_token_provided = true;
-    }
+    TALER_MHD_parse_request_arg_auto (connection,
+                                      "h_contract",
+                                      &god->h_contract_terms,
+                                      god->h_contract_provided);
+    TALER_MHD_parse_request_arg_auto (connection,
+                                      "token",
+                                      &god->claim_token,
+                                      god->claim_token_provided);
     god->session_id = MHD_lookup_connection_value (connection,
                                                    MHD_GET_ARGUMENT_KIND,
                                                    "session_id");
@@ -936,89 +876,62 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
       }
     }
 
-
-    /* process timeout_ms argument */
+    TALER_MHD_parse_request_timeout (connection,
+                                     &god->sc.long_poll_timeout);
+    /* If HTML is requested, we never actually long poll. Makes no sense */
+    if ( (! god->generate_html) &&
+         (GNUNET_TIME_absolute_is_future (god->sc.long_poll_timeout)) )
     {
-      const char *long_poll_timeout_ms;
-
-      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
-                                                          MHD_GET_ARGUMENT_KIND,
-                                                          "timeout_ms");
-      if (NULL != long_poll_timeout_ms)
+      if (god->sc.awaiting_refund ||
+          god->sc.awaiting_refund_obtained)
       {
-        unsigned int timeout_ms;
-        char dummy;
+        struct TMH_OrderPayEventP refund_eh = {
+          .header.size = htons (sizeof (refund_eh)),
+          .header.type = htons (god->sc.awaiting_refund_obtained
+                                    ? TALER_DBEVENT_MERCHANT_REFUND_OBTAINED
+                                    : TALER_DBEVENT_MERCHANT_ORDER_REFUND),
+          .merchant_pub = hc->instance->merchant_pub
+        };
 
-        if (1 != sscanf (long_poll_timeout_ms,
-                         "%u%c",
-                         &timeout_ms,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms (must be non-negative number)");
-        }
-        /* If HTML is requested, we never long poll. Makes no sense */
-        if (! god->generate_html)
-        {
-          struct GNUNET_TIME_Relative timeout;
+        GNUNET_CRYPTO_hash (god->order_id,
+                            strlen (god->order_id),
+                            &refund_eh.h_order_id);
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Subscribing %p to refunds on %s\n",
+                    god,
+                    god->order_id);
+        god->refund_eh
+          = TMH_db->event_listen (
+              TMH_db->cls,
+              &refund_eh.header,
+              GNUNET_TIME_absolute_get_remaining (
+                god->sc.long_poll_timeout),
+              &resume_by_event,
+              god);
+      }
+      {
+        struct TMH_OrderPayEventP pay_eh = {
+          .header.size = htons (sizeof (pay_eh)),
+          .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
+          .merchant_pub = hc->instance->merchant_pub
+        };
 
-          timeout = GNUNET_TIME_relative_multiply (
-            GNUNET_TIME_UNIT_MILLISECONDS,
-            timeout_ms);
-          god->sc.long_poll_timeout
-            = GNUNET_TIME_relative_to_absolute (timeout);
-          if (! GNUNET_TIME_relative_is_zero (timeout))
-          {
-            if (god->sc.awaiting_refund ||
-                god->sc.awaiting_refund_obtained)
-            {
-              struct TMH_OrderPayEventP refund_eh = {
-                .header.size = htons (sizeof (refund_eh)),
-                .header.type = htons (god->sc.awaiting_refund_obtained
-                                      ? TALER_DBEVENT_MERCHANT_REFUND_OBTAINED
-                                      : TALER_DBEVENT_MERCHANT_ORDER_REFUND),
-                .merchant_pub = hc->instance->merchant_pub
-              };
-
-              GNUNET_CRYPTO_hash (god->order_id,
-                                  strlen (god->order_id),
-                                  &refund_eh.h_order_id);
-              GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                          "Subscribing %p to refunds on %s\n",
-                          god,
-                          god->order_id);
-              god->refund_eh = TMH_db->event_listen (TMH_db->cls,
-                                                     &refund_eh.header,
-                                                     timeout,
-                                                     &resume_by_event,
-                                                     god);
-            }
-            {
-              struct TMH_OrderPayEventP pay_eh = {
-                .header.size = htons (sizeof (pay_eh)),
-                .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
-                .merchant_pub = hc->instance->merchant_pub
-              };
-
-              GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                          "Subscribing to payments on %s\n",
-                          god->order_id);
-              GNUNET_CRYPTO_hash (god->order_id,
-                                  strlen (god->order_id),
-                                  &pay_eh.h_order_id);
-              god->pay_eh = TMH_db->event_listen (TMH_db->cls,
-                                                  &pay_eh.header,
-                                                  timeout,
-                                                  &resume_by_event,
-                                                  god);
-            }
-          } /* end of timeout non-zero */
-        } /* end of HTML generation NOT requested */
-      } /* end of timeout_ms argument provided */
-    } /* end of timeout_ms argument handling */
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Subscribing to payments on %s\n",
+                    god->order_id);
+        GNUNET_CRYPTO_hash (god->order_id,
+                            strlen (god->order_id),
+                            &pay_eh.h_order_id);
+        god->pay_eh
+          = TMH_db->event_listen (
+              TMH_db->cls,
+              &pay_eh.header,
+              GNUNET_TIME_absolute_get_remaining (
+                god->sc.long_poll_timeout),
+              &resume_by_event,
+              god);
+      }
+    }     /* end of do long-polling / no HTML */
 
   } /* end of first-time initialization / sanity checks */
 
@@ -1070,10 +983,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   if (NULL != god->contract_terms)
   {
     contract_available = true;
-
-    if (GNUNET_YES == GNUNET_is_zero (&god->h_contract_terms))
+    if (GNUNET_YES ==
+        GNUNET_is_zero (&god->h_contract_terms))
     {
-
       if (GNUNET_OK !=
           TALER_JSON_contract_hash (god->contract_terms,
                                     &god->h_contract_terms))
@@ -1084,11 +996,9 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                            TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
                                            "contract terms");
       }
-
     }
     else
     {
-
       struct TALER_PrivateContractHashP h;
 
       if (GNUNET_OK !=
@@ -1113,9 +1023,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
           TALER_EC_MERCHANT_GENERIC_CONTRACT_HASH_DOES_NOT_MATCH_ORDER,
           NULL);
       }
-
     }
-
   }
 
   if (contract_available)
@@ -1165,21 +1073,47 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   } /* end unclaimed order logic */
 
   GNUNET_assert (NULL != god->contract_terms);
-  merchant_base_url = json_string_value (json_object_get (god->contract_terms,
-                                                          "merchant_base_url"));
-  if (NULL == merchant_base_url)
+  if (! god->contract_parsed)
   {
-    GNUNET_break (0);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_MERCHANT_GENERIC_DB_CONTRACT_CONTENT_INVALID,
-                                       order_id);
+    struct GNUNET_JSON_Specification espec[] = {
+      TALER_JSON_spec_amount_any ("amount",
+                                  &god->contract_total),
+      TALER_JSON_spec_web_url ("merchant_base_url",
+                               &god->merchant_base_url),
+      GNUNET_JSON_spec_mark_optional (
+        /* this one does NOT have to be a Web URL! */
+        GNUNET_JSON_spec_string ("fulfillment_url",
+                                 &god->fulfillment_url),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        TALER_JSON_spec_web_url ("public_reorder_url",
+                                 &god->public_reorder_url),
+        NULL),
+      GNUNET_JSON_spec_end ()
+    };
+    enum GNUNET_GenericReturnValue res;
+    const char *ename;
+    unsigned int eline;
+
+    res = GNUNET_JSON_parse (god->contract_terms,
+                             espec,
+                             &ename,
+                             &eline);
+    if (GNUNET_OK != res)
+    {
+      GNUNET_break (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Failed to parse contract %s in DB at field %s\n",
+                  order_id,
+                  ename);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_MERCHANT_GENERIC_DB_CONTRACT_CONTENT_INVALID,
+                                         order_id);
+    }
+    god->contract_parsed = true;
   }
 
-  if (NULL == god->fulfillment_url)
-    god->fulfillment_url = json_string_value (json_object_get (
-                                                god->contract_terms,
-                                                "fulfillment_url"));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Token match: %d, contract_available: %d, contract match: %d, claimed: %d\n",
               token_match,
@@ -1187,7 +1121,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
               contract_match,
               god->claimed);
 
-  if (claim_token_provided && ! token_match)
+  if (god->claim_token_provided && ! token_match)
   {
     /* Authentication provided but wrong. */
     GNUNET_break_op (0);
@@ -1197,7 +1131,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                        "authentication with claim token provided but wrong");
   }
 
-  if (h_contract_provided && ! contract_match)
+  if (god->h_contract_provided && ! contract_match)
   {
     /* Authentication provided but wrong. */
     GNUNET_break_op (0);
@@ -1210,15 +1144,11 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
   if (! (token_match ||
          contract_match) )
   {
-    const char *public_reorder_url;
 
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Neither claim token nor contract matched\n");
-    public_reorder_url = json_string_value (json_object_get (
-                                              god->contract_terms,
-                                              "public_reorder_url"));
     /* Client has no rights to this order */
-    if (NULL == public_reorder_url)
+    if (NULL == god->public_reorder_url)
     {
       /* We cannot give the client a new order, just fail */
       if (! GNUNET_is_zero (&god->h_contract_terms))
@@ -1260,7 +1190,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
       GNUNET_break (MHD_YES ==
                     MHD_add_response_header (reply,
                                              MHD_HTTP_HEADER_LOCATION,
-                                             public_reorder_url));
+                                             god->public_reorder_url));
       ret = MHD_queue_response (connection,
                                 MHD_HTTP_FOUND,
                                 reply);
@@ -1272,7 +1202,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
       connection,
       MHD_HTTP_ACCEPTED,
       GNUNET_JSON_pack_string ("public_reorder_url",
-                               public_reorder_url));
+                               god->public_reorder_url));
   }
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -1372,13 +1302,25 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
     }
   }
 
+  if ( (god->sc.awaiting_refund) &&
+       (GNUNET_OK !=
+        TALER_amount_cmp_currency (&god->contract_total,
+                                   &god->sc.refund_expected)) )
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_CONFLICT,
+                                       TALER_EC_MERCHANT_GENERIC_CURRENCY_MISMATCH,
+                                       god->contract_total.currency);
+  }
+
   /* At this point, we know the contract was paid. Let's check for
      refunds. First, clear away refunds found from previous invocations. */
   GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_set_zero (TMH_currency,
+                 TALER_amount_set_zero (god->contract_total.currency,
                                         &god->refund_amount));
   GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_set_zero (TMH_currency,
+                 TALER_amount_set_zero (god->contract_total.currency,
                                         &god->refund_taken));
   qs = TMH_db->lookup_refunds_detailed (TMH_db->cls,
                                         hc->instance->settings.id,
@@ -1393,7 +1335,14 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
                                        TALER_EC_GENERIC_DB_FETCH_FAILED,
                                        "lookup_refunds_detailed");
   }
-
+  if (god->bad_refund_currency_in_db)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                       "currency mix-up between contract price and refunds in database");
+  }
   if ( ((god->sc.awaiting_refund) &&
         ( (! god->refunded) ||
           (1 != TALER_amount_cmp (&god->refund_amount,
@@ -1406,7 +1355,8 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
     struct GNUNET_TIME_Relative remaining;
 
     remaining = GNUNET_TIME_absolute_get_remaining (god->sc.long_poll_timeout);
-    if (! GNUNET_TIME_relative_is_zero (remaining))
+    if ( (! GNUNET_TIME_relative_is_zero (remaining)) &&
+         (! god->generate_html) )
     {
       /* yes, indeed suspend */
       if (god->sc.awaiting_refund)
@@ -1432,7 +1382,7 @@ TMH_get_orders_ID (const struct TMH_RequestHandler *rh,
       char *uri;
 
       GNUNET_assert (NULL != god->contract_terms);
-      uri = make_taler_refund_uri (merchant_base_url,
+      uri = make_taler_refund_uri (god->merchant_base_url,
                                    order_id);
       if (NULL == uri)
       {
