@@ -67,7 +67,7 @@ TMH_make_order_status_url (struct MHD_Connection *con,
                            const char *session_id,
                            const char *instance_id,
                            struct TALER_ClaimTokenP *claim_token,
-                           struct GNUNET_HashCode *h_contract);
+                           struct TALER_PrivateContractHashP *h_contract);
 
 
 /**
