@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2022 Taler Systems SA
+  (C) 2022-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -21,6 +21,7 @@
  * @file taler-merchant-httpd_post-using-templates.c
  * @brief implementing POST /using-templates request handling
  * @author Priscilla HUANG
+ * @author Christian Grothoff
  */
 #include "platform.h"
 #include "taler-merchant-httpd_post-using-templates.h"
@@ -29,13 +30,49 @@
 #include <taler/taler_json_lib.h>
 
 
+/**
+ * Our context.
+ */
+struct UseContext
+{
+  /**
+   * Internal handler context we are passing into the
+   * POST /private/orders handler.
+   */
+  struct TMH_HandlerContext ihc;
+
+  /**
+   * Our template details from the DB.
+   */
+  struct TALER_MERCHANTDB_TemplateDetails etp;
+
+};
+
+
+/**
+ * Clean up a `struct UseContext *`
+ *
+ * @param cls a `struct UseContext *`
+ */
+static void
+cleanup_use_context (void *cls)
+{
+  struct UseContext *uc = cls;
+
+  TALER_MERCHANTDB_template_details_free (&uc->etp);
+  if (NULL != uc->ihc.cc)
+    uc->ihc.cc (uc->ihc.ctx);
+  json_decref (uc->ihc.request_body);
+  GNUNET_free (uc);
+}
+
+
 MHD_RESULT
 TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
                              struct MHD_Connection *connection,
                              struct TMH_HandlerContext *hc)
 {
   struct TMH_MerchantInstance *mi = hc->instance;
-  MHD_RESULT mret;
   const char *template_id = hc->infix;
   const char *summary = NULL;
   const char *fulfillment_url = NULL;
@@ -44,7 +81,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
   bool no_amount;
   json_t *fake_body;
   bool no_summary;
-  struct TALER_MERCHANTDB_TemplateDetails etp;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("summary",
@@ -57,6 +93,15 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
       &no_amount),
     GNUNET_JSON_spec_end ()
   };
+  struct UseContext *uc = hc->ctx;
+
+  if (NULL == uc)
+  {
+    uc = GNUNET_new (struct UseContext);
+    hc->ctx = uc;
+    hc->cc = &cleanup_use_context;
+    uc->ihc.instance = hc->instance;
+  }
 
   {
     enum GNUNET_GenericReturnValue res;
@@ -79,32 +124,35 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
     qs = TMH_db->lookup_template (TMH_db->cls,
                                   mi->settings.id,
                                   template_id,
-                                  &etp);
+                                  &uc->etp);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
       /* Clean up and fail hard */
       GNUNET_break (0);
       GNUNET_JSON_parse_free (spec);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                         TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                         NULL);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_EC_GENERIC_DB_FETCH_FAILED,
+        NULL);
     case GNUNET_DB_STATUS_SOFT_ERROR:
       /* this should be impossible (single select) */
       GNUNET_break (0);
       GNUNET_JSON_parse_free (spec);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                         TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                         NULL);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_EC_GENERIC_DB_FETCH_FAILED,
+        NULL);
     case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
       /* template not found! */
       GNUNET_JSON_parse_free (spec);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_NOT_FOUND,
-                                         TALER_EC_MERCHANT_GENERIC_TEMPLATE_UNKNOWN,
-                                         template_id);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_NOT_FOUND,
+        TALER_EC_MERCHANT_GENERIC_TEMPLATE_UNKNOWN,
+        template_id);
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       /* all good */
       break;
@@ -118,7 +166,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
     struct GNUNET_TIME_Relative pay_duration;
     struct TALER_Amount tamount;
     bool no_tamount;
-
     struct GNUNET_JSON_Specification tspec[] = {
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string ("summary",
@@ -141,22 +188,22 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
       const char *err_name;
       unsigned int err_line;
 
-      res = GNUNET_JSON_parse (etp.template_contract,
+      res = GNUNET_JSON_parse (uc->etp.template_contract,
                                tspec,
                                &err_name,
                                &err_line);
       if (GNUNET_OK != res)
       {
         GNUNET_break (0);
-        json_dumpf (etp.template_contract,
+        json_dumpf (uc->etp.template_contract,
                     stderr,
                     JSON_INDENT (2));
-        TALER_MERCHANTDB_template_details_free (&etp);
         GNUNET_JSON_parse_free (spec);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                           TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                           err_name);
+        return TALER_MHD_reply_with_error (
+          connection,
+          MHD_HTTP_INTERNAL_SERVER_ERROR,
+          TALER_EC_GENERIC_DB_FETCH_FAILED,
+          err_name);
       }
     }
 
@@ -164,7 +211,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
          (! no_tamount) )
     {
       GNUNET_JSON_parse_free (spec);
-      TALER_MERCHANTDB_template_details_free (&etp);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_CONFLICT,
@@ -175,7 +221,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
     if (no_amount && no_tamount)
     {
       GNUNET_JSON_parse_free (spec);
-      TALER_MERCHANTDB_template_details_free (&etp);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_CONFLICT,
@@ -187,7 +232,6 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
          (NULL != tsummary) )
     {
       GNUNET_JSON_parse_free (spec);
-      TALER_MERCHANTDB_template_details_free (&etp);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_CONFLICT,
@@ -199,48 +243,43 @@ TMH_post_using_templates_ID (const struct TMH_RequestHandler *rh,
          (NULL == tsummary) )
     {
       GNUNET_JSON_parse_free (spec);
-      TALER_MERCHANTDB_template_details_free (&etp);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_CONFLICT,
-                                         TALER_EC_MERCHANT_POST_USING_TEMPLATES_NO_SUMMARY,
-                                         NULL);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_CONFLICT,
+        TALER_EC_MERCHANT_POST_USING_TEMPLATES_NO_SUMMARY,
+        NULL);
     }
     no_summary = (NULL == summary);
     fake_body = GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_object_steal ("order",
-                                     GNUNET_JSON_PACK (
-                                       TALER_JSON_pack_amount ("amount",
-                                                               no_amount ?
-                                                               &tamount :
-                                                               &amount),
-                                       GNUNET_JSON_pack_string ("summary",
-                                                                no_summary ?
-                                                                tsummary :
-                                                                summary),
-                                       GNUNET_JSON_pack_allow_null (
-                                         GNUNET_JSON_pack_string (
-                                           "fulfillment_url",
-                                           fulfillment_url)),
-                                       GNUNET_JSON_pack_allow_null (
-                                         GNUNET_JSON_pack_string (
-                                           "fulfillment_message",
-                                           fulfillment_message))
-                                       ))
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("otp_id",
+                                 uc->etp.otp_id)),
+      GNUNET_JSON_pack_object_steal (
+        "order",
+        GNUNET_JSON_PACK (
+          TALER_JSON_pack_amount ("amount",
+                                  no_amount ?
+                                  &tamount :
+                                  &amount),
+          GNUNET_JSON_pack_string ("summary",
+                                   no_summary ?
+                                   tsummary :
+                                   summary),
+          GNUNET_JSON_pack_allow_null (
+            GNUNET_JSON_pack_string (
+              "fulfillment_url",
+              fulfillment_url)),
+          GNUNET_JSON_pack_allow_null (
+            GNUNET_JSON_pack_string (
+              "fulfillment_message",
+              fulfillment_message))
+          ))
       );
   }
-  {
-    struct TMH_HandlerContext fake_hc = {
-      .request_body = fake_body,
-      .instance = hc->instance
-    };
 
-    mret = TMH_private_post_orders_with_pos_secrets (NULL,   /* not even used */
-                                                     connection,
-                                                     &fake_hc,
-                                                     etp.pos_key,
-                                                     etp.pos_algorithm);
-  }
-  TALER_MERCHANTDB_template_details_free (&etp);
-  json_decref (fake_body);
-  return mret;
+  uc->ihc.request_body = fake_body;
+  return TMH_private_post_orders (
+    NULL,                                                  /* not even used */
+    connection,
+    &uc->ihc);
 }
