@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2019-2021 Taler Systems SA
+  (C) 2019-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -74,6 +74,9 @@ add_instance (void *cls,
                    GNUNET_JSON_PACK (
                      GNUNET_JSON_pack_string ("name",
                                               mi->settings.name),
+                     GNUNET_JSON_pack_string (
+                       "user_type",
+                       TALER_KYCLOGIC_kyc_user_type2s (mi->settings.ut)),
                      GNUNET_JSON_pack_allow_null (
                        GNUNET_JSON_pack_string ("website",
                                                 mi->settings.website)),
