@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2017-2022 Taler Systems SA
+  (C) 2017-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -73,7 +73,7 @@ struct TransferQuery
   /**
    * Handle for ongoing exchange operation.
    */
-  struct TMH_EXCHANGES_FindOperation *fo;
+  struct TMH_EXCHANGES_KeysOperation *fo;
 
   /**
    * Overall request this TQ belongs with.
@@ -285,6 +285,19 @@ struct GetOrderRequestContext
    */
   bool transfer_status_requested;
 
+  /**
+   * Set to true if our database (incorrectly) has refunds
+   * in a different currency than the currency of the
+   * original payment for the order.
+   */
+  bool refund_currency_mismatch;
+
+  /**
+   * Set to true if our database (incorrectly) has deposits
+   * in a different currency than the currency of the
+   * original payment for the order.
+   */
+  bool deposit_currency_mismatch;
 };
 
 
@@ -340,7 +353,7 @@ gorc_resume (struct GetOrderRequestContext *gorc,
   {
     if (NULL != tq->fo)
     {
-      TMH_EXCHANGES_find_exchange_cancel (tq->fo);
+      TMH_EXCHANGES_keys4exchange_cancel (tq->fo);
       tq->fo = NULL;
     }
     if (NULL != tq->dgh)
@@ -468,7 +481,7 @@ deposit_get_cb (void *cls,
 
       qs = TMH_db->insert_deposit_to_transfer (TMH_db->cls,
                                                tq->deposit_serial,
-                                               &dr->details.success);
+                                               &dr->details.ok);
       if (qs < 0)
       {
         gorc_report (gorc,
@@ -484,10 +497,33 @@ deposit_get_cb (void *cls,
         return;
       }
       /* Compute total amount *wired* */
+      if ( (GNUNET_OK !=
+            TALER_amount_cmp_currency (
+              &gorc->deposits_total,
+              &dr->details.ok.coin_contribution)) ||
+           (GNUNET_OK !=
+            TALER_amount_cmp_currency (
+              &gorc->deposit_fees_total,
+              &tq->deposit_fee)) )
+      {
+        /* something very wrong in our database ... */
+        GNUNET_break (0);
+        gorc_report (gorc,
+                     TALER_EC_GENERIC_DB_FETCH_FAILED,
+                     &tq->coin_pub,
+                     NULL);
+        GNUNET_free (tq->exchange_url);
+        GNUNET_free (tq);
+        if (NULL == gorc->tq_head)
+          gorc_resume (gorc,
+                       0,
+                       TALER_EC_NONE);
+        return;
+      }
       if (0 >
           TALER_amount_add (&gorc->deposits_total,
                             &gorc->deposits_total,
-                            &dr->details.success.coin_contribution))
+                            &dr->details.ok.coin_contribution))
       {
         gorc_report (gorc,
                      TALER_EC_MERCHANT_PRIVATE_GET_ORDERS_ID_AMOUNT_ARITHMETIC_FAILURE,
@@ -586,29 +622,24 @@ deposit_get_cb (void *cls,
 
 
 /**
- * Function called with the result of a #TMH_EXCHANGES_find_exchange()
+ * Function called with the result of a #TMH_EXCHANGES_keys4exchange()
  * operation.
  *
  * @param cls closure with a `struct GetOrderRequestContext *`
- * @param hr HTTP response details
- * @param eh handle to the exchange context
- * @param payto_uri payto://-URI of the exchange
- * @param wire_fee current applicable wire fee for dealing with @a eh, NULL if not available
- * @param exchange_trusted true if this exchange is trusted by config
+ * @param keys keys of the exchange
+ * @param exchange representation of the exchange
  */
 static void
 exchange_found_cb (void *cls,
-                   const struct TALER_EXCHANGE_HttpResponse *hr,
-                   struct TALER_EXCHANGE_Handle *eh,
-                   const char *payto_uri,
-                   const struct TALER_Amount *wire_fee,
-                   bool exchange_trusted)
+                   struct TALER_EXCHANGE_Keys *keys,
+                   struct TMH_Exchange *exchange)
 {
   struct TransferQuery *tq = cls;
   struct GetOrderRequestContext *gorc = tq->gorc;
 
+  (void) exchange;
   tq->fo = NULL;
-  if (NULL == hr)
+  if (NULL == keys)
   {
     /* failed */
     GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
@@ -621,28 +652,17 @@ exchange_found_cb (void *cls,
                  TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT);
     return;
   }
-  if (NULL == eh)
-  {
-    /* failed */
-    GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
-                                 gorc->tq_tail,
-                                 tq);
-    GNUNET_free (tq->exchange_url);
-    GNUNET_free (tq);
-    gorc->exchange_hc = hr->http_status;
-    gorc->exchange_ec = hr->ec;
-    gorc_resume (gorc,
-                 MHD_HTTP_BAD_GATEWAY,
-                 TALER_EC_MERCHANT_GENERIC_EXCHANGE_CONNECT_FAILURE);
-    return;
-  }
-  tq->dgh = TALER_EXCHANGE_deposits_get (eh,
-                                         &gorc->hc->instance->merchant_priv,
-                                         &tq->h_wire,
-                                         &gorc->h_contract_terms,
-                                         &tq->coin_pub,
-                                         &deposit_get_cb,
-                                         tq);
+  tq->dgh = TALER_EXCHANGE_deposits_get (
+    TMH_curl_ctx,
+    tq->exchange_url,
+    keys,
+    &gorc->hc->instance->merchant_priv,
+    &tq->h_wire,
+    &gorc->h_contract_terms,
+    &tq->coin_pub,
+    GNUNET_TIME_UNIT_ZERO,
+    &deposit_get_cb,
+    tq);
   if (NULL == tq->dgh)
   {
     GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
@@ -696,9 +716,8 @@ deposit_cb (void *cls,
   tq->h_wire = *h_wire;
   tq->amount_with_fee = *amount_with_fee;
   tq->deposit_fee = *deposit_fee;
-  tq->fo = TMH_EXCHANGES_find_exchange (exchange_url,
-                                        NULL,
-                                        GNUNET_NO,
+  tq->fo = TMH_EXCHANGES_keys4exchange (exchange_url,
+                                        false,
                                         &exchange_found_cb,
                                         tq);
   if (NULL == tq->fo)
@@ -792,12 +811,29 @@ process_refunds_cb (void *cls,
         GNUNET_memcmp (&tq->coin_pub,
                        coin_pub))
     {
+      if (GNUNET_OK !=
+          TALER_amount_cmp_currency (
+            &gorc->deposit_fees_total,
+            &tq->deposit_fee))
+      {
+        gorc->refund_currency_mismatch = true;
+        return;
+      }
+         
       GNUNET_assert (0 <=
                      TALER_amount_subtract (&gorc->deposit_fees_total,
                                             &gorc->deposit_fees_total,
                                             &tq->deposit_fee));
     }
   }
+  if (GNUNET_OK !=
+      TALER_amount_cmp_currency (
+        &gorc->refund_amount,
+        refund_amount))
+  {
+    gorc->refund_currency_mismatch = true;
+    return;
+  }
   GNUNET_assert (0 <=
                  TALER_amount_add (&gorc->refund_amount,
                                    &gorc->refund_amount,
@@ -821,18 +857,31 @@ process_refunds_cb (void *cls,
  *        @a wtid over the total amount happened?
  */
 static void
-process_transfer_details (void *cls,
-                          const struct TALER_WireTransferIdentifierRawP *wtid,
-                          const char *exchange_url,
-                          struct GNUNET_TIME_Timestamp execution_time,
-                          const struct TALER_Amount *deposit_value,
-                          const struct TALER_Amount *deposit_fee,
-                          bool transfer_confirmed)
+process_transfer_details (
+  void *cls,
+  const struct TALER_WireTransferIdentifierRawP *wtid,
+  const char *exchange_url,
+  struct GNUNET_TIME_Timestamp execution_time,
+  const struct TALER_Amount *deposit_value,
+  const struct TALER_Amount *deposit_fee,
+  bool transfer_confirmed)
 {
   struct GetOrderRequestContext *gorc = cls;
   json_t *wire_details = gorc->wire_details;
   struct TALER_Amount wired;
 
+  if ( (GNUNET_OK !=
+        TALER_amount_cmp_currency (&gorc->deposits_total,
+                                   deposit_value)) ||
+       (GNUNET_OK !=
+        TALER_amount_cmp_currency (&gorc->deposit_fees_total,
+                                   deposit_fee)) )
+  {
+    GNUNET_break (0);
+    gorc->deposit_currency_mismatch = true;
+    return;
+  }
+       
   /* Compute total amount *wired* */
   GNUNET_assert (0 <
                  TALER_amount_add (&gorc->deposits_total,
@@ -909,83 +958,53 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
         gorc->transfer_status_requested = true;
     }
 
-    /* process 'timeout_ms' argument */
+    TALER_MHD_parse_request_timeout (connection,
+                                     &gorc->sc.long_poll_timeout);
+    if (GNUNET_TIME_absolute_is_future (gorc->sc.long_poll_timeout))
     {
-      const char *long_poll_timeout_s;
+      struct TMH_OrderPayEventP pay_eh = {
+        .header.size = htons (sizeof (pay_eh)),
+        .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
+        .merchant_pub = hc->instance->merchant_pub
+      };
 
-      long_poll_timeout_s = MHD_lookup_connection_value (connection,
-                                                         MHD_GET_ARGUMENT_KIND,
-                                                         "timeout_ms");
-      if (NULL != long_poll_timeout_s)
+      GNUNET_CRYPTO_hash (hc->infix,
+                          strlen (hc->infix),
+                          &pay_eh.h_order_id);
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Subscribing to payment triggers for %p\n",
+                  gorc);
+      gorc->eh = TMH_db->event_listen (
+        TMH_db->cls,
+        &pay_eh.header,
+        GNUNET_TIME_absolute_get_remaining (gorc->sc.long_poll_timeout),
+        &resume_by_event,
+        gorc);
+      if ( (NULL != gorc->session_id) &&
+           (NULL != gorc->fulfillment_url) )
       {
-        unsigned int timeout_ms;
-        char dummy;
-        struct GNUNET_TIME_Relative timeout;
+        struct TMH_SessionEventP session_eh = {
+          .header.size = htons (sizeof (session_eh)),
+          .header.type = htons (TALER_DBEVENT_MERCHANT_SESSION_CAPTURED),
+          .merchant_pub = hc->instance->merchant_pub
+        };
 
-        if (1 != sscanf (long_poll_timeout_s,
-                         "%u%c",
-                         &timeout_ms,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms must be non-negative number");
-        }
-        timeout = GNUNET_TIME_relative_multiply (
-          GNUNET_TIME_UNIT_MILLISECONDS,
-          timeout_ms);
-        gorc->sc.long_poll_timeout
-          = GNUNET_TIME_relative_to_absolute (timeout);
-        if (! GNUNET_TIME_relative_is_zero (timeout))
-        {
-          struct TMH_OrderPayEventP pay_eh = {
-            .header.size = htons (sizeof (pay_eh)),
-            .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
-            .merchant_pub = hc->instance->merchant_pub
-          };
-
-          GNUNET_CRYPTO_hash (hc->infix,
-                              strlen (hc->infix),
-                              &pay_eh.h_order_id);
-          GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                      "Subscribing to payment triggers for %p\n",
-                      gorc);
-          gorc->eh = TMH_db->event_listen (TMH_db->cls,
-                                           &pay_eh.header,
-                                           timeout,
-                                           &resume_by_event,
-                                           gorc);
-          if ( (NULL != gorc->session_id) &&
-               (NULL != gorc->fulfillment_url) )
-          {
-            struct TMH_SessionEventP session_eh = {
-              .header.size = htons (sizeof (session_eh)),
-              .header.type = htons (TALER_DBEVENT_MERCHANT_SESSION_CAPTURED),
-              .merchant_pub = hc->instance->merchant_pub
-            };
-
-            GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                        "Subscribing to session triggers for %p\n",
-                        gorc);
-            GNUNET_CRYPTO_hash (gorc->session_id,
-                                strlen (gorc->session_id),
-                                &session_eh.h_session_id);
-            GNUNET_CRYPTO_hash (gorc->fulfillment_url,
-                                strlen (gorc->fulfillment_url),
-                                &session_eh.h_fulfillment_url);
-            gorc->session_eh = TMH_db->event_listen (TMH_db->cls,
-                                                     &session_eh.header,
-                                                     timeout,
-                                                     &resume_by_event,
-                                                     gorc);
-          }
-        }
-      }
-      else
-      {
-        gorc->sc.long_poll_timeout = GNUNET_TIME_UNIT_ZERO_ABS;
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Subscribing to session triggers for %p\n",
+                    gorc);
+        GNUNET_CRYPTO_hash (gorc->session_id,
+                            strlen (gorc->session_id),
+                            &session_eh.h_session_id);
+        GNUNET_CRYPTO_hash (gorc->fulfillment_url,
+                            strlen (gorc->fulfillment_url),
+                            &session_eh.h_fulfillment_url);
+        gorc->session_eh
+          = TMH_db->event_listen (
+              TMH_db->cls,
+              &session_eh.header,
+              GNUNET_TIME_absolute_get_remaining (gorc->sc.long_poll_timeout),
+              &resume_by_event,
+              gorc);
       }
     }
   } /* end first-time per-request initialization */
@@ -1085,9 +1104,8 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
      from the contract terms! */
   {
     struct GNUNET_JSON_Specification spec[] = {
-      TALER_JSON_spec_amount ("amount",
-                              TMH_currency,
-                              &gorc->contract_amount),
+      TALER_JSON_spec_amount_any ("amount",
+                                  &gorc->contract_amount),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string ("fulfillment_url",
                                  &gorc->fulfillment_url),
@@ -1281,10 +1299,10 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
     /* suspend connection, wait for exchange to check wire transfer status there */
     gorc->transfer_status_requested = false;   /* only try ONCE */
     GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TMH_currency,
+                   TALER_amount_set_zero (gorc->contract_amount.currency,
                                           &gorc->deposits_total));
     GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TMH_currency,
+                   TALER_amount_set_zero (gorc->contract_amount.currency,
                                           &gorc->deposit_fees_total));
     TMH_db->lookup_deposits_by_order (TMH_db->cls,
                                       gorc->order_serial,
@@ -1363,7 +1381,7 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
   /* Accumulate refunds, if any. */
   {
     GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TMH_currency,
+                   TALER_amount_set_zero (gorc->contract_amount.currency,
                                           &gorc->refund_amount));
     qs = TMH_db->lookup_refunds_detailed (TMH_db->cls,
                                           hc->instance->settings.id,
@@ -1379,6 +1397,14 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                                        TALER_EC_GENERIC_DB_FETCH_FAILED,
                                        "detailed refunds");
   }
+  if (gorc->refund_currency_mismatch)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                       "refunds in different currency than original order price");
+  }
 
   /* Generate final reply, including wire details if we have them */
   {
@@ -1386,10 +1412,10 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
     char *order_status_url;
 
     GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TMH_currency,
+                   TALER_amount_set_zero (gorc->contract_amount.currency,
                                           &gorc->deposits_total));
     GNUNET_assert (GNUNET_OK ==
-                   TALER_amount_set_zero (TMH_currency,
+                   TALER_amount_set_zero (gorc->contract_amount.currency,
                                           &gorc->deposit_fees_total));
     qs = TMH_db->lookup_transfer_details_by_order (TMH_db->cls,
                                                    gorc->order_serial,
@@ -1403,6 +1429,14 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                                          TALER_EC_GENERIC_DB_FETCH_FAILED,
                                          "transfer details");
     }
+    if (gorc->deposit_currency_mismatch)
+    {
+      GNUNET_break (0);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                         "deposits in different currency than original order price");
+    }
 
     if (! wired)
     {
