@@ -68,16 +68,13 @@ TMH_private_get_templates_ID (const struct TMH_RequestHandler *rh,
       MHD_HTTP_OK,
       GNUNET_JSON_pack_string ("template_description",
                                tp.template_description),
-      GNUNET_JSON_pack_uint64 ("pos_algorithm",
-                               tp.pos_algorithm),
       GNUNET_JSON_pack_allow_null (
-        GNUNET_JSON_pack_string ("pos_key",
-                                 tp.pos_key)),
+        GNUNET_JSON_pack_string ("otp_id",
+                                 tp.otp_id)),
       GNUNET_JSON_pack_object_steal ("template_contract",
                                      tp.template_contract));
     GNUNET_free (tp.template_description);
-    GNUNET_free (tp.pos_key);
-
+    GNUNET_free (tp.otp_id);
     return ret;
   }
 }
