@@ -121,7 +121,8 @@ TMH_private_patch_products_ID (const struct TMH_RequestHandler *rh,
                              (const char **) &pd.description),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("description_i18n",
-                             &pd.description_i18n)),
+                             &pd.description_i18n),
+      NULL),
     GNUNET_JSON_spec_string ("unit",
                              (const char **) &pd.unit),
     TALER_JSON_spec_amount ("price",
@@ -129,21 +130,30 @@ TMH_private_patch_products_ID (const struct TMH_RequestHandler *rh,
                             &pd.price),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("image",
-                               (const char **) &pd.image)),
+                               (const char **) &pd.image),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("taxes",
-                             &pd.taxes)),
+                             &pd.taxes),
+      NULL),
     GNUNET_JSON_spec_int64 ("total_stock",
                             &total_stock),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_uint64 ("total_lost",
-                               &pd.total_lost)),
+                               &pd.total_lost),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("address",
-                             &pd.address)),
+                             &pd.address),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("next_restock",
-                                     &pd.next_restock)),
+      GNUNET_JSON_spec_timestamp ("next_restock",
+                                  &pd.next_restock),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint32 ("minimum_age",
+                               &pd.minimum_age),
+      NULL),
     GNUNET_JSON_spec_end ()
   };
 
