@@ -43,8 +43,8 @@ TMH_private_post_products_ID_lock (const struct TMH_RequestHandler *rh,
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("lock_uuid",
                              &uuids),
-    TALER_JSON_spec_relative_time ("duration",
-                                   &duration),
+    GNUNET_JSON_spec_relative_time ("duration",
+                                    &duration),
     GNUNET_JSON_spec_uint32 ("quantity",
                              &quantity),
     GNUNET_JSON_spec_end ()
@@ -71,7 +71,7 @@ TMH_private_post_products_ID_lock (const struct TMH_RequestHandler *rh,
                              product_id,
                              &uuid,
                              quantity,
-                             GNUNET_TIME_relative_to_absolute (duration));
+                             GNUNET_TIME_relative_to_timestamp (duration));
   switch (qs)
   {
   case GNUNET_DB_STATUS_HARD_ERROR:
