@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2021, 2022 Taler Systems SA
+  (C) 2021-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -16,7 +16,6 @@
   License along with TALER; see the file COPYING.  If not,
   see <http://www.gnu.org/licenses/>
 */
-
 /**
  * @file taler-merchant-httpd_private-post-reserves.c
  * @brief implementing POST /reserves request handling
@@ -25,6 +24,7 @@
 #include "platform.h"
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_private-post-reserves.h"
+#include "taler-merchant-httpd_helper.h"
 #include "taler-merchant-httpd_reserves.h"
 #include <taler/taler_json_lib.h>
 
@@ -74,19 +74,25 @@ struct PostReserveContext
   const char *exchange_url;
 
   /**
-   * URI of the exchange where the payment needs to be made to.
+   * Wire method the client wants to use for the payment.
    */
-  char *payto_uri;
+  const char *wire_method;
 
   /**
-   * Handle for contacting the exchange.
+   * Array of accounts that could be used.
    */
-  struct TMH_EXCHANGES_FindOperation *fo;
+  json_t *accounts;
 
   /**
-   * Task run on timeout.
+   * Handle for contacting the exchange for /keys.
    */
-  struct GNUNET_SCHEDULER_Task *timeout_task;
+  struct TMH_EXCHANGES_KeysOperation *fo;
+
+  /**
+   * Master public key of the exchange matching
+   * @e exchange_url.
+   */
+  struct TALER_MasterPublicKeyP master_pub;
 
   /**
    * Initial balance of the reserve.
@@ -116,6 +122,11 @@ struct PostReserveContext
    * without a response due to shutdown.
    */
   enum GNUNET_GenericReturnValue suspended;
+
+  /**
+   * True if we already force reloaded /keys.
+   */
+  bool force_reload;
 };
 
 
@@ -144,11 +155,6 @@ TMH_force_rc_resume ()
        rc = rcn)
   {
     rcn = rc->next;
-    if (NULL != rc->timeout_task)
-    {
-      GNUNET_SCHEDULER_cancel (rc->timeout_task);
-      rc->timeout_task = NULL;
-    }
     if (GNUNET_YES == rc->suspended)
     {
       rc->suspended = GNUNET_SYSERR;
@@ -159,7 +165,7 @@ TMH_force_rc_resume ()
     }
     if (NULL != rc->fo)
     {
-      TMH_EXCHANGES_find_exchange_cancel (rc->fo);
+      TMH_EXCHANGES_keys4exchange_cancel (rc->fo);
       rc->fo = NULL;
     }
   }
@@ -178,122 +184,105 @@ reserve_context_cleanup (void *cls)
 
   if (NULL != rc->fo)
   {
-    TMH_EXCHANGES_find_exchange_cancel (rc->fo);
+    TMH_EXCHANGES_keys4exchange_cancel (rc->fo);
     rc->fo = NULL;
   }
-  if (NULL != rc->timeout_task)
-  {
-    GNUNET_SCHEDULER_cancel (rc->timeout_task);
-    rc->timeout_task = NULL;
-  }
   GNUNET_assert (GNUNET_YES != rc->suspended);
-  GNUNET_free (rc->payto_uri);
+  json_decref (rc->accounts);
   GNUNET_free (rc);
 }
 
 
 /**
- * Function called with the result of a #TMH_EXCHANGES_find_exchange()
- * operation.
+ * Resume request handling.
  *
- * @param cls closure with our `struct PostReserveContext *`
- * @param hr HTTP response details
- * @param payto_uri URI of the exchange for the wire transfer, NULL on errors
- * @param eh handle to the exchange context
- * @param wire_fee current applicable wire fee for dealing with @a eh, NULL if not available
- * @param exchange_trusted true if this exchange is trusted by config
+ * @param[in,out] rc request to resume
  */
 static void
-handle_exchange (void *cls,
-                 const struct TALER_EXCHANGE_HttpResponse *hr,
-                 struct TALER_EXCHANGE_Handle *eh,
-                 const char *payto_uri,
-                 const struct TALER_Amount *wire_fee,
-                 bool exchange_trusted)
+resume_request (struct PostReserveContext *rc)
 {
-  struct PostReserveContext *rc = cls;
-  const struct TALER_EXCHANGE_Keys *keys;
-
-  rc->fo = NULL;
-  if (NULL != rc->timeout_task)
-  {
-    GNUNET_SCHEDULER_cancel (rc->timeout_task);
-    rc->timeout_task = NULL;
-  }
   rc->suspended = GNUNET_NO;
   MHD_resume_connection (rc->connection);
   GNUNET_CONTAINER_DLL_remove (rc_head,
                                rc_tail,
                                rc);
-  if (NULL == hr)
-  {
-    rc->ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT;
-    rc->http_status = MHD_HTTP_GATEWAY_TIMEOUT;
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-    return;
-  }
-  if (NULL == eh)
-  {
-    rc->ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_CONNECT_FAILURE;
-    rc->http_status = MHD_HTTP_BAD_GATEWAY;
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-    return;
-  }
-  keys = TALER_EXCHANGE_get_keys (eh);
-  if (NULL == keys)
-  {
-    rc->ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_KEYS_FAILURE;
-    rc->http_status = MHD_HTTP_BAD_GATEWAY;
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-    return;
-  }
-  if (MHD_HTTP_OK != hr->http_status)
-  {
-    rc->ec = hr->ec;
-    rc->http_status = hr->http_status;
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-    return;
-  }
-  if (NULL == payto_uri)
-  {
-    rc->ec = TALER_EC_MERCHANT_PRIVATE_POST_RESERVES_UNSUPPORTED_WIRE_METHOD;
-    rc->http_status = MHD_HTTP_CONFLICT;
-    TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
-    return;
-  }
-  rc->reserve_expiration
-    = GNUNET_TIME_relative_to_timestamp (keys->reserve_closing_delay);
-  rc->payto_uri = GNUNET_strdup (payto_uri);
-  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
+  TALER_MHD_daemon_trigger (); /* we resumed, kick MHD */
 }
 
 
 /**
- * Handle a timeout for the processing of the wire request.
+ * Function called with the result of a #TMH_EXCHANGES_keys4exchange()
+ * operation.
  *
- * @param cls closure
+ * @param cls closure with our `struct PostReserveContext *`
+ * @param keys exchange keys
+ * @param exchange representation of the exchange
  */
 static void
-handle_exchange_timeout (void *cls)
+handle_exchange (void *cls,
+                 struct TALER_EXCHANGE_Keys *keys,
+                 struct TMH_Exchange *exchange)
 {
   struct PostReserveContext *rc = cls;
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Resuming POST /private/reserves with error after timeout\n");
-  rc->timeout_task = NULL;
-  if (NULL != rc->fo)
+  rc->fo = NULL;
+  if (NULL == keys)
   {
-    TMH_EXCHANGES_find_exchange_cancel (rc->fo);
-    rc->fo = NULL;
+    GNUNET_break_op (0);
+    rc->ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_KEYS_FAILURE;
+    rc->http_status = MHD_HTTP_GATEWAY_TIMEOUT;
+    resume_request (rc);
+    return;
   }
-  rc->suspended = GNUNET_NO;
-  MHD_resume_connection (rc->connection);
-  GNUNET_CONTAINER_DLL_remove (rc_head,
-                               rc_tail,
-                               rc);
-  rc->ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT;
-  rc->http_status = MHD_HTTP_GATEWAY_TIMEOUT;
-  TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
+  if (! keys->rewards_allowed)
+  {
+    if (! rc->force_reload)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Forcing %s/keys reload (rewards not allowed)\n",
+                  rc->exchange_url);
+      rc->force_reload = true;
+      rc->fo = TMH_EXCHANGES_keys4exchange (rc->exchange_url,
+                                            true,
+                                            &handle_exchange,
+                                            rc);
+      return;
+    }
+    rc->ec = TALER_EC_MERCHANT_PRIVATE_POST_RESERVES_REWARDS_NOT_ALLOWED;
+    rc->http_status = MHD_HTTP_CONFLICT;
+    resume_request (rc);
+    return;
+  }
+  rc->master_pub = keys->master_pub;
+  rc->reserve_expiration
+    = GNUNET_TIME_relative_to_timestamp (keys->reserve_closing_delay);
+  rc->accounts = TMH_exchange_accounts_by_method (
+    &rc->master_pub,
+    rc->wire_method);
+  if ( (NULL == rc->accounts) ||
+       (0 == json_array_size (rc->accounts)) )
+  {
+    if (! rc->force_reload)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Forcing %s/keys reload (no accounts)\n",
+                  rc->exchange_url);
+      rc->force_reload = true;
+      rc->fo = TMH_EXCHANGES_keys4exchange (rc->exchange_url,
+                                            true,
+                                            &handle_exchange,
+                                            rc);
+      return;
+    }
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Wire method `%s' not supported\n",
+                rc->wire_method);
+    rc->ec = TALER_EC_MERCHANT_PRIVATE_POST_RESERVES_UNSUPPORTED_WIRE_METHOD;
+    rc->http_status = MHD_HTTP_CONFLICT;
+    resume_request (rc);
+    return;
+  }
+  resume_request (rc);
 }
 
 
@@ -308,8 +297,6 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
   GNUNET_assert (NULL != mi);
   if (NULL == rc)
   {
-    const char *wire_method;
-
     rc = GNUNET_new (struct PostReserveContext);
     rc->connection = connection;
     rc->hc = hc;
@@ -317,17 +304,17 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
     hc->cc = &reserve_context_cleanup;
 
     {
-      enum GNUNET_GenericReturnValue res;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_string ("exchange_url",
+        TALER_JSON_spec_web_url ("exchange_url",
                                  &rc->exchange_url),
         GNUNET_JSON_spec_string ("wire_method",
-                                 &wire_method),
-        TALER_JSON_spec_amount ("initial_balance",
-                                TMH_currency,
-                                &rc->initial_balance),
+                                 &rc->wire_method),
+        TALER_JSON_spec_amount_any ("initial_balance",
+                                    &rc->initial_balance),
         GNUNET_JSON_spec_end ()
       };
+      enum GNUNET_GenericReturnValue res;
+
       res = TALER_MHD_parse_json_data (connection,
                                        hc->request_body,
                                        spec);
@@ -336,15 +323,10 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
                ? MHD_YES
                : MHD_NO;
     }
-    rc->fo = TMH_EXCHANGES_find_exchange (rc->exchange_url,
-                                          wire_method,
-                                          GNUNET_NO,
+    rc->fo = TMH_EXCHANGES_keys4exchange (rc->exchange_url,
+                                          false,
                                           &handle_exchange,
                                           rc);
-    rc->timeout_task
-      = GNUNET_SCHEDULER_add_delayed (EXCHANGE_GENERIC_TIMEOUT,
-                                      &handle_exchange_timeout,
-                                      rc);
     rc->suspended = GNUNET_YES;
     GNUNET_CONTAINER_DLL_insert (rc_head,
                                  rc_tail,
@@ -354,15 +336,16 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
   }
   if (GNUNET_SYSERR == rc->suspended)
     return MHD_NO; /* we are in shutdown */
-
-  GNUNET_assert (GNUNET_NO == rc->suspended);
-  if (NULL == rc->payto_uri)
+  if (TALER_EC_NONE != rc->ec)
   {
     return TALER_MHD_reply_with_error (connection,
                                        rc->http_status,
                                        rc->ec,
                                        NULL);
   }
+  GNUNET_assert (GNUNET_NO == rc->suspended);
+  GNUNET_assert (! GNUNET_TIME_absolute_is_zero (
+                   rc->reserve_expiration.abs_time));
   {
     struct TALER_ReservePublicKeyP reserve_pub;
     struct TALER_ReservePrivateKeyP reserve_priv;
@@ -375,8 +358,8 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
                                  mi->settings.id,
                                  &reserve_priv,
                                  &reserve_pub,
+                                 &rc->master_pub,
                                  rc->exchange_url,
-                                 rc->payto_uri,
                                  &rc->initial_balance,
                                  rc->reserve_expiration);
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
@@ -394,8 +377,8 @@ TMH_private_post_reserves (const struct TMH_RequestHandler *rh,
       MHD_HTTP_OK,
       GNUNET_JSON_pack_data_auto ("reserve_pub",
                                   &reserve_pub),
-      GNUNET_JSON_pack_string ("payto_uri",
-                               rc->payto_uri));
+      GNUNET_JSON_pack_array_incref ("accounts",
+                                     rc->accounts));
   }
 }
 
