@@ -23,10 +23,6 @@ BEGIN;
 -- Unlike the other SQL files, it SHOULD be updated to reflect the
 -- latest requirements for dropping tables.
 
--- Unregister patch (0003.sql)
-SELECT _v.unregister_patch('merchant-0004');
-SELECT _v.unregister_patch('merchant-0003');
-SELECT _v.unregister_patch('merchant-0002');
 SELECT _v.unregister_patch('merchant-0001');
 
 
