@@ -18,21 +18,9 @@
 BEGIN;
 
 -- Check patch versioning is in place.
-SELECT _v.register_patch('merchant-0002', NULL, NULL);
+-- SELECT _v.register_patch('merchant-0002', NULL, NULL);
 
 SET search_path TO merchant;
 
-ALTER TABLE merchant_instances
-  ADD COLUMN website VARCHAR,
-  ADD COLUMN email VARCHAR,
-  ADD COLUMN logo BYTEA;
-
-COMMENT ON COLUMN merchant_instances.website
-  IS 'merchant site URL';
-COMMENT ON COLUMN merchant_instances.email
-  IS 'email';
-COMMENT ON COLUMN merchant_instances.logo
-  IS 'data image url';
-
 -- Complete transaction
 COMMIT;
