@@ -18,13 +18,9 @@
 BEGIN;
 
 -- Check patch versioning is in place.
-SELECT _v.register_patch('merchant-0003', NULL, NULL);
+-- SELECT _v.register_patch('merchant-0003', NULL, NULL);
 
 SET search_path TO merchant;
 
-ALTER TABLE merchant_exchange_wire_fees
-  DROP COLUMN wad_fee_val,
-  DROP COLUMN wad_fee_frac;
-
 -- Complete transaction
 COMMIT;
