@@ -37,13 +37,12 @@ TALER_MERCHANTDB_product_details_free (
 }
 
 
-
 void
 TALER_MERCHANTDB_template_details_free (
   struct TALER_MERCHANTDB_TemplateDetails *tp)
 {
   GNUNET_free (tp->template_description);
-  GNUNET_free (tp->pos_key);
+  GNUNET_free (tp->otp_id);
   json_decref (tp->template_contract);
 }
 
@@ -59,6 +58,7 @@ TALER_MERCHANTDB_webhook_details_free (
   GNUNET_free (wb->body_template);
 }
 
+
 void
 TALER_MERCHANTDB_pending_webhook_details_free (
   struct TALER_MERCHANTDB_PendingWebhookDetails *pwb)
@@ -70,5 +70,4 @@ TALER_MERCHANTDB_pending_webhook_details_free (
 }
 
 
-
 /* end of merchantdb_helper.c */
