@@ -19,15 +19,16 @@
  * @brief Implementation of the DELETE /reserves/$RESERVE_PUB request of the merchant's HTTP API
  * @author Jonathan Buchanan
  */
- #include "platform.h"
- #include <curl/curl.h>
- #include <jansson.h>
- #include <microhttpd.h> /* just for HTTP status codes */
- #include <gnunet/gnunet_util_lib.h>
- #include <gnunet/gnunet_curl_lib.h>
- #include "taler_merchant_service.h"
- #include <taler/taler_json_lib.h>
- #include <taler/taler_signatures.h>
+#include "platform.h"
+#include <curl/curl.h>
+#include <jansson.h>
+#include <microhttpd.h> /* just for HTTP status codes */
+#include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_curl_lib.h>
+#include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
+#include <taler/taler_json_lib.h>
+#include <taler/taler_signatures.h>
 
 
 /**
@@ -180,11 +181,7 @@ reserve_delete (struct GNUNET_CURL_Context *ctx,
   {
     CURL *eh;
 
-    eh = curl_easy_init ();
-    GNUNET_assert (CURLE_OK ==
-                   curl_easy_setopt (eh,
-                                     CURLOPT_URL,
-                                     rdh->url));
+    eh = TALER_MERCHANT_curl_easy_get_ (rdh->url);
     GNUNET_assert (CURLE_OK ==
                    curl_easy_setopt (eh,
                                      CURLOPT_CUSTOMREQUEST,
