@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018, 2020 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -34,12 +34,12 @@
  * Which version of the Taler protocol is implemented
  * by this library?  Used to determine compatibility.
  */
-#define MERCHANT_PROTOCOL_CURRENT 3
+#define MERCHANT_PROTOCOL_CURRENT 5
 
 /**
  * How many configs are we backwards compatible with?
  */
-#define MERCHANT_PROTOCOL_AGE 1
+#define MERCHANT_PROTOCOL_AGE 0
 
 
 /**
@@ -90,9 +90,9 @@ handle_config_finished (void *cls,
 {
   struct TALER_MERCHANT_ConfigGetHandle *vgh = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .http_status = (unsigned int) response_code,
-    .reply = json
+  struct TALER_MERCHANT_ConfigResponse cr = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
   };
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -104,76 +104,93 @@ handle_config_finished (void *cls,
   {
   case MHD_HTTP_OK:
     {
-      struct TALER_MERCHANT_ConfigInformation vi;
-      enum TALER_MERCHANT_VersionCompatibility vc =
-        TALER_MERCHANT_VC_PROTOCOL_ERROR;
+      const json_t *jcs;
+      struct TALER_JSON_ProtocolVersion pv;
       struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_object_const ("currencies",
+                                       &jcs),
         GNUNET_JSON_spec_string ("currency",
-                                 &vi.currency),
+                                 &cr.details.ok.ci.currency),
+        TALER_JSON_spec_version ("version",
+                                 &pv),
         GNUNET_JSON_spec_string ("version",
-                                 &vi.version),
+                                 &cr.details.ok.ci.version),
         GNUNET_JSON_spec_end ()
       };
 
+      cr.details.ok.compat = TALER_MERCHANT_VC_PROTOCOL_ERROR;
       if (GNUNET_OK !=
           GNUNET_JSON_parse (json,
                              spec,
                              NULL, NULL))
       {
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        cr.hr.http_status = 0;
+        cr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
       }
       else
       {
-        unsigned int age;
-        unsigned int revision;
-        unsigned int current;
-
-        if (3 != sscanf (vi.version,
-                         "%u:%u:%u",
-                         &current,
-                         &revision,
-                         &age))
+        cr.details.ok.compat = TALER_MERCHANT_VC_MATCH;
+        if (MERCHANT_PROTOCOL_CURRENT < pv.current)
         {
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+          cr.details.ok.compat |= TALER_MERCHANT_VC_NEWER;
+          if (MERCHANT_PROTOCOL_CURRENT < pv.current - pv.age)
+            cr.details.ok.compat |= TALER_MERCHANT_VC_INCOMPATIBLE;
         }
-        else
+        if (MERCHANT_PROTOCOL_CURRENT > pv.current)
         {
-          vc = TALER_MERCHANT_VC_MATCH;
-          if (MERCHANT_PROTOCOL_CURRENT < current)
-          {
-            vc |= TALER_MERCHANT_VC_NEWER;
-            if (MERCHANT_PROTOCOL_CURRENT < current - age)
-              vc |= TALER_MERCHANT_VC_INCOMPATIBLE;
-          }
-          if (MERCHANT_PROTOCOL_CURRENT > current)
-          {
-            vc |= TALER_MERCHANT_VC_OLDER;
-            if (MERCHANT_PROTOCOL_CURRENT - MERCHANT_PROTOCOL_AGE > current)
-              vc |= TALER_MERCHANT_VC_INCOMPATIBLE;
-          }
+          cr.details.ok.compat |= TALER_MERCHANT_VC_OLDER;
+          if (MERCHANT_PROTOCOL_CURRENT - MERCHANT_PROTOCOL_AGE > pv.current)
+            cr.details.ok.compat |= TALER_MERCHANT_VC_INCOMPATIBLE;
         }
       }
-      vgh->cb (vgh->cb_cls,
-               &hr,
-               &vi,
-               vc);
+      {
+        unsigned int nspec = json_object_size (jcs);
+        struct TALER_CurrencySpecification *cspecs;
+        unsigned int off = 0;
+        json_t *obj;
+        const char *curr;
+
+        cspecs = GNUNET_new_array (nspec,
+                                   struct TALER_CurrencySpecification);
+        cr.details.ok.num_cspecs = nspec;
+        cr.details.ok.cspecs = cspecs;
+        json_object_foreach ((json_t *) jcs, curr, obj)
+        {
+          struct TALER_CurrencySpecification *cs = &cspecs[off++];
+          struct GNUNET_JSON_Specification cspec[] = {
+            TALER_JSON_spec_currency_specification (curr,
+                                                    cs),
+            GNUNET_JSON_spec_end ()
+          };
+
+          if (GNUNET_OK !=
+              GNUNET_JSON_parse (jcs,
+                                 cspec,
+                                 NULL, NULL))
+          {
+            cr.hr.http_status = 0;
+            cr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+            break;
+          }
+        }
+        vgh->cb (vgh->cb_cls,
+                 &cr);
+        TALER_CONFIG_free_currencies (nspec,
+                                      cspecs);
+      }
       TALER_MERCHANT_config_get_cancel (vgh);
       return;
     }
   default:
     /* unexpected response code */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    cr.hr.ec = TALER_JSON_get_error_code (json);
+    cr.hr.hint = TALER_JSON_get_error_hint (json);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) cr.hr.ec);
     vgh->cb (vgh->cb_cls,
-             &hr,
-             NULL,
-             TALER_MERCHANT_VC_PROTOCOL_ERROR);
+             &cr);
     break;
   }
   TALER_MERCHANT_config_get_cancel (vgh);
