@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017, 2020 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -81,23 +81,23 @@ handle_reserves_get_finished (void *cls,
 {
   struct TALER_MERCHANT_ReservesGetHandle *rgh = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .http_status = (unsigned int) response_code,
-    .reply = json
+  struct TALER_MERCHANT_ReservesGetResponse rgr = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
   };
 
   rgh->job = NULL;
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    rgr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
-      json_t *reserves;
+      const json_t *reserves;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_json ("reserves",
-                               &reserves),
+        GNUNET_JSON_spec_array_const ("reserves",
+                                      &reserves),
         GNUNET_JSON_spec_end ()
       };
 
@@ -107,11 +107,10 @@ handle_reserves_get_finished (void *cls,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        rgr.hr.http_status = 0;
+        rgr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
         break;
       }
-      else
       {
         size_t rds_length;
         struct TALER_MERCHANT_ReserveSummary *rds;
@@ -119,14 +118,6 @@ handle_reserves_get_finished (void *cls,
         unsigned int i;
         bool ok;
 
-        if (! json_is_array (reserves))
-        {
-          GNUNET_break_op (0);
-          GNUNET_JSON_parse_free (spec);
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
-          break;
-        }
         rds_length = json_array_size (reserves);
         rds = GNUNET_new_array (rds_length,
                                 struct TALER_MERCHANT_ReserveSummary);
@@ -169,14 +160,14 @@ handle_reserves_get_finished (void *cls,
           GNUNET_break_op (0);
           GNUNET_free (rds);
           GNUNET_JSON_parse_free (spec);
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+          rgr.hr.http_status = 0;
+          rgr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
           break;
         }
+        rgr.details.ok.reserves = rds;
+        rgr.details.ok.reserves_length = rds_length;
         rgh->cb (rgh->cb_cls,
-                 &hr,
-                 rds_length,
-                 rds);
+                 &rgr);
         GNUNET_free (rds);
         GNUNET_JSON_parse_free (spec);
         TALER_MERCHANT_reserves_get_cancel (rgh);
@@ -184,45 +175,43 @@ handle_reserves_get_finished (void *cls,
       }
     }
   case MHD_HTTP_UNAUTHORIZED:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    rgr.hr.ec = TALER_JSON_get_error_code (json);
+    rgr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, merchant says we need to authenticate. */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    rgr.hr.ec = TALER_JSON_get_error_code (json);
+    rgr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
   default:
     /* unexpected response code */
     GNUNET_break_op (0);
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &rgr.hr);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
-    response_code = 0;
+                (int) rgr.hr.ec);
     break;
   }
   rgh->cb (rgh->cb_cls,
-           &hr,
-           0,
-           NULL);
+           &rgr);
   TALER_MERCHANT_reserves_get_cancel (rgh);
 }
 
 
 struct TALER_MERCHANT_ReservesGetHandle *
-TALER_MERCHANT_reserves_get (struct GNUNET_CURL_Context *ctx,
-                             const char *backend_url,
-                             struct GNUNET_TIME_Timestamp after,
-                             enum TALER_EXCHANGE_YesNoAll active,
-                             enum TALER_EXCHANGE_YesNoAll failures,
-                             TALER_MERCHANT_ReservesGetCallback cb,
-                             void *cb_cls)
+TALER_MERCHANT_reserves_get (
+  struct GNUNET_CURL_Context *ctx,
+  const char *backend_url,
+  struct GNUNET_TIME_Timestamp after,
+  enum TALER_EXCHANGE_YesNoAll active,
+  enum TALER_EXCHANGE_YesNoAll failures,
+  TALER_MERCHANT_ReservesGetCallback cb,
+  void *cb_cls)
 {
   struct TALER_MERCHANT_ReservesGetHandle *rgh;
   CURL *eh;
