@@ -81,21 +81,21 @@ handle_unpaid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
   struct GNUNET_JSON_Specification spec[] = {
     TALER_JSON_spec_amount_any (
       "total_amount",
-      &osr->details.success.details.unpaid.contract_amount),
+      &osr->details.ok.details.unpaid.contract_amount),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string (
         "already_paid_order_id",
-        &osr->details.success.details.unpaid.already_paid_order_id),
+        &osr->details.ok.details.unpaid.already_paid_order_id),
       NULL),
     GNUNET_JSON_spec_string (
       "taler_pay_uri",
-      &osr->details.success.details.unpaid.taler_pay_uri),
+      &osr->details.ok.details.unpaid.taler_pay_uri),
     GNUNET_JSON_spec_string (
       "summary",
-      &osr->details.success.details.unpaid.summary),
+      &osr->details.ok.details.unpaid.summary),
     GNUNET_JSON_spec_timestamp (
       "creation_time",
-      &osr->details.success.details.unpaid.creation_time),
+      &osr->details.ok.details.unpaid.creation_time),
     GNUNET_JSON_spec_end ()
   };
 
@@ -111,7 +111,7 @@ handle_unpaid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
               osr);
     return;
   }
-  osr->details.success.status = TALER_MERCHANT_OSC_UNPAID;
+  osr->details.ok.status = TALER_MERCHANT_OSC_UNPAID;
   omgh->cb (omgh->cb_cls,
             osr);
 }
@@ -130,9 +130,9 @@ handle_claimed (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
                 struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_json ("contract_terms",
-                           (json_t **) &osr->details.success.details.claimed.
-                           contract_terms),
+    GNUNET_JSON_spec_object_const (
+      "contract_terms",
+      &osr->details.ok.details.claimed.contract_terms),
     GNUNET_JSON_spec_end ()
   };
 
@@ -148,10 +148,9 @@ handle_claimed (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
               osr);
     return;
   }
-  osr->details.success.status = TALER_MERCHANT_OSC_CLAIMED;
+  osr->details.ok.status = TALER_MERCHANT_OSC_CLAIMED;
   omgh->cb (omgh->cb_cls,
             osr);
-  GNUNET_JSON_parse_free (spec);
 }
 
 
@@ -167,35 +166,34 @@ static void
 handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
              struct TALER_MERCHANT_OrderStatusResponse *osr)
 {
-  uint32_t ec32;
   uint32_t hc32;
-  json_t *wire_details;
-  json_t *wire_reports;
-  json_t *refund_details;
+  const json_t *wire_details;
+  const json_t *wire_reports;
+  const json_t *refund_details;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_bool ("refunded",
-                           &osr->details.success.details.paid.refunded),
+                           &osr->details.ok.details.paid.refunded),
     GNUNET_JSON_spec_bool ("refund_pending",
-                           &osr->details.success.details.paid.refund_pending),
+                           &osr->details.ok.details.paid.refund_pending),
     GNUNET_JSON_spec_bool ("wired",
-                           &osr->details.success.details.paid.wired),
+                           &osr->details.ok.details.paid.wired),
     TALER_JSON_spec_amount_any ("deposit_total",
-                                &osr->details.success.details.paid.deposit_total),
-    GNUNET_JSON_spec_uint32 ("exchange_code",
-                             &ec32),
+                                &osr->details.ok.details.paid.deposit_total),
+    TALER_JSON_spec_ec ("exchange_code",
+                        &osr->details.ok.details.paid.exchange_ec),
     GNUNET_JSON_spec_uint32 ("exchange_http_status",
                              &hc32),
     TALER_JSON_spec_amount_any ("refund_amount",
-                                &osr->details.success.details.paid.refund_amount),
-    GNUNET_JSON_spec_json (
+                                &osr->details.ok.details.paid.refund_amount),
+    GNUNET_JSON_spec_object_const (
       "contract_terms",
-      (json_t **) &osr->details.success.details.paid.contract_terms),
-    GNUNET_JSON_spec_json ("wire_details",
-                           &wire_details),
-    GNUNET_JSON_spec_json ("wire_reports",
-                           &wire_reports),
-    GNUNET_JSON_spec_json ("refund_details",
-                           &refund_details),
+      &osr->details.ok.details.paid.contract_terms),
+    GNUNET_JSON_spec_array_const ("wire_details",
+                                  &wire_details),
+    GNUNET_JSON_spec_array_const ("wire_reports",
+                                  &wire_reports),
+    GNUNET_JSON_spec_array_const ("refund_details",
+                                  &refund_details),
     GNUNET_JSON_spec_end ()
   };
 
@@ -211,29 +209,16 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
               osr);
     return;
   }
-  if (! (json_is_array (wire_details) &&
-         json_is_array (wire_reports) &&
-         json_is_array (refund_details) &&
-         json_is_object (osr->details.success.details.paid.contract_terms)) )
-  {
-    GNUNET_break_op (0);
-    osr->hr.http_status = 0;
-    osr->hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-    omgh->cb (omgh->cb_cls,
-              osr);
-    GNUNET_JSON_parse_free (spec);
-    return;
-  }
-  osr->details.success.status = TALER_MERCHANT_OSC_PAID;
-  osr->details.success.details.paid.exchange_ec = (enum TALER_ErrorCode) ec32;
-  osr->details.success.details.paid.exchange_hc = (unsigned int) hc32;
+  osr->details.ok.status = TALER_MERCHANT_OSC_PAID;
+  
+  osr->details.ok.details.paid.exchange_hc = (unsigned int) hc32;
   {
     unsigned int wts_len = json_array_size (wire_details);
     unsigned int wrs_len = json_array_size (wire_reports);
     unsigned int ref_len = json_array_size (refund_details);
-    struct TALER_MERCHANT_WireTransfer wts[wts_len];
-    struct TALER_MERCHANT_WireReport wrs[wrs_len];
-    struct TALER_MERCHANT_RefundOrderDetail ref[ref_len];
+    struct TALER_MERCHANT_WireTransfer wts[GNUNET_NZL (wts_len)];
+    struct TALER_MERCHANT_WireReport wrs[GNUNET_NZL (wrs_len)];
+    struct TALER_MERCHANT_RefundOrderDetail ref[GNUNET_NZL (ref_len)];
 
     for (unsigned int i = 0; i<wts_len; i++)
     {
@@ -241,7 +226,7 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
       const json_t *w = json_array_get (wire_details,
                                         i);
       struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_string ("exchange_url",
+        TALER_JSON_spec_web_url ("exchange_url",
                                  &wt->exchange_url),
         GNUNET_JSON_spec_fixed_auto ("wtid",
                                      &wt->wtid),
@@ -264,7 +249,6 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
         osr->hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         omgh->cb (omgh->cb_cls,
                   osr);
-        GNUNET_JSON_parse_free (spec);
         return;
       }
     }
@@ -273,16 +257,14 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
     {
       struct TALER_MERCHANT_WireReport *wr = &wrs[i];
       const json_t *w = json_array_get (wire_reports, i);
-      uint32_t c32;
-      uint32_t eec32;
       uint32_t ehs32;
       struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_uint32 ("code",
-                                 &c32),
+        TALER_JSON_spec_ec ("code",
+                            &wr->code),
         GNUNET_JSON_spec_string ("hint",
                                  &wr->hint),
-        GNUNET_JSON_spec_uint32 ("exchange_code",
-                                 &eec32),
+        TALER_JSON_spec_ec ("exchange_code",
+                            &wr->hr.ec),
         GNUNET_JSON_spec_uint32 ("exchange_http_status",
                                  &ehs32),
         GNUNET_JSON_spec_fixed_auto ("coin_pub",
@@ -300,11 +282,8 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
         osr->hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         omgh->cb (omgh->cb_cls,
                   osr);
-        GNUNET_JSON_parse_free (spec);
         return;
       }
-      wr->code = (enum TALER_ErrorCode) c32;
-      wr->hr.ec = (enum TALER_ErrorCode) eec32;
       wr->hr.http_status = (unsigned int) ehs32;
     }
 
@@ -333,21 +312,19 @@ handle_paid (struct TALER_MERCHANT_OrderMerchantGetHandle *omgh,
         osr->hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         omgh->cb (omgh->cb_cls,
                   osr);
-        GNUNET_JSON_parse_free (spec);
         return;
       }
     }
 
-    osr->details.success.details.paid.wts = wts;
-    osr->details.success.details.paid.wts_len = wts_len;
-    osr->details.success.details.paid.wrs = wrs;
-    osr->details.success.details.paid.wrs_len = wrs_len;
-    osr->details.success.details.paid.refunds = ref;
-    osr->details.success.details.paid.refunds_len = ref_len;
+    osr->details.ok.details.paid.wts = wts;
+    osr->details.ok.details.paid.wts_len = wts_len;
+    osr->details.ok.details.paid.wrs = wrs;
+    osr->details.ok.details.paid.wrs_len = wrs_len;
+    osr->details.ok.details.paid.refunds = ref;
+    osr->details.ok.details.paid.refunds_len = ref_len;
     omgh->cb (omgh->cb_cls,
               osr);
   }
-  GNUNET_JSON_parse_free (spec);
 }
 
 
