@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016, 2020 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -86,26 +86,27 @@ handle_post_reserves_finished (void *cls,
 {
   struct TALER_MERCHANT_PostReservesHandle *prh = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_HttpResponse hr = {
-    .http_status = (unsigned int) response_code,
-    .reply = json
+  struct TALER_MERCHANT_PostReservesResponse prr = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
   };
 
   prh->job = NULL;
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    prr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
-      const char *payto_uri;
-      struct TALER_ReservePublicKeyP reserve_pub;
+      const json_t *accounts;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_fixed_auto ("reserve_pub",
-                                     &reserve_pub),
-        GNUNET_JSON_spec_string ("payto_uri",
-                                 &payto_uri),
+        GNUNET_JSON_spec_fixed_auto (
+          "reserve_pub",
+          &prr.details.ok.reserve_pub),
+        GNUNET_JSON_spec_array_const (
+          "accounts",
+          &accounts),
         GNUNET_JSON_spec_end ()
       };
 
@@ -115,16 +116,31 @@ handle_post_reserves_finished (void *cls,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        prr.hr.http_status = 0;
+        prr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
         break;
       }
-      else
+
       {
+        unsigned int accounts_len = json_array_size (accounts);
+        struct TALER_EXCHANGE_WireAccount was[GNUNET_NZL (accounts_len)];
+
+        prr.details.ok.accounts = was;
+        prr.details.ok.accounts_len = accounts_len;
+        if (GNUNET_OK !=
+            TALER_EXCHANGE_parse_accounts (NULL,
+                                           accounts,
+                                           accounts_len,
+                                           was))
+        {
+          GNUNET_break_op (0);
+          prr.hr.http_status = 0;
+          prr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        }
         prh->cb (prh->cb_cls,
-                 &hr,
-                 &reserve_pub,
-                 payto_uri);
+                 &prr);
+        TALER_EXCHANGE_free_accounts (accounts_len,
+                                      was);
         GNUNET_JSON_parse_free (spec);
         TALER_MERCHANT_reserves_post_cancel (prh);
         return;
@@ -133,8 +149,8 @@ handle_post_reserves_finished (void *cls,
   case MHD_HTTP_ACCEPTED:
     break;
   case MHD_HTTP_UNAUTHORIZED:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    prr.hr.ec = TALER_JSON_get_error_code (json);
+    prr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, merchant says we need to authenticate. */
     break;
   case MHD_HTTP_NOT_FOUND:
@@ -142,31 +158,29 @@ handle_post_reserves_finished (void *cls,
        happen, we should pass the JSON reply to the application */
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Did not find any data\n");
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    prr.hr.ec = TALER_JSON_get_error_code (json);
+    prr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    prr.hr.ec = TALER_JSON_get_error_code (json);
+    prr.hr.hint = TALER_JSON_get_error_hint (json);
     break;
   default:
     /* unexpected response code */
     GNUNET_break_op (0);
     TALER_MERCHANT_parse_error_details_ (json,
                                          response_code,
-                                         &hr);
+                                         &prr.hr);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) prr.hr.ec);
     break;
   }
   prh->cb (prh->cb_cls,
-           &hr,
-           NULL,
-           NULL);
+           &prr);
   TALER_MERCHANT_reserves_post_cancel (prh);
 }
 
