@@ -116,11 +116,11 @@ handle_wallet_get_order_finished (void *cls,
       };
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_bool ("refunded",
-                               &owgr.details.success.refunded),
+                               &owgr.details.ok.refunded),
         GNUNET_JSON_spec_bool ("refund_pending",
-                               &owgr.details.success.refund_pending),
+                               &owgr.details.ok.refund_pending),
         TALER_JSON_spec_amount_any ("refund_amount",
-                                    &owgr.details.success.refund_amount),
+                                    &owgr.details.ok.refund_amount),
         GNUNET_JSON_spec_end ()
       };
 
