@@ -22,6 +22,9 @@
 
 set -eu
 
+unset XDG_DATA_HOME
+unset XDG_CONFIG_HOME
+
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
     echo $1
