@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2022 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -43,8 +43,6 @@
  */
 #define CONFIG_FILE "test_kyc_api.conf"
 
-#define PAYTO_I1 "payto://x-taler-bank/localhost/3?receiver-name=3"
-
 /**
  * Exchange base URL.  Could also be taken from config.
  */
@@ -53,43 +51,33 @@
 /**
  * Payto URI of the customer (payer).
  */
-static char *payer_payto;
+static const char *payer_payto;
 
 /**
  * Payto URI of the exchange (escrow account).
  */
-static char *exchange_payto;
+static const char *exchange_payto;
 
 /**
  * Payto URI of the merchant (receiver).
  */
-static char *merchant_payto;
+static const char *merchant_payto;
 
 /**
- * Configuration of the bank.
+ * Credentials for the test.
  */
-static struct TALER_TESTING_BankConfiguration bc;
-
-/**
- * Configuration of the exchange.
- */
-static struct TALER_TESTING_ExchangeConfiguration ec;
+static struct TALER_TESTING_Credentials cred;
 
 /**
  * Merchant base URL.
  */
-static char *merchant_url;
+static const char *merchant_url;
 
 /**
  * Merchant instance "i1a" base URL.
  */
 static char *merchant_url_i1a;
 
-/**
- * Merchant process.
- */
-static struct GNUNET_OS_Process *merchantd;
-
 /**
  * Account number of the exchange at the bank.
  */
@@ -100,11 +88,6 @@ static struct GNUNET_OS_Process *merchantd;
  */
 #define USER_ACCOUNT_NAME "62"
 
-/**
- * Account number of some other user.
- */
-#define USER_ACCOUNT_NAME2 "63"
-
 /**
  * Account number used by the merchant
  */
@@ -112,7 +95,7 @@ static struct GNUNET_OS_Process *merchantd;
 
 
 /**
- * Execute the taler-exchange-aggregator, closer and transfer commands with
+ * Execute the taler-exchange-aggregator and transfer commands with
  * our configuration file.
  *
  * @param label label to use for the command.
@@ -136,7 +119,7 @@ cmd_transfer_to_exchange (const char *label,
 {
   return TALER_TESTING_cmd_admin_add_incoming (label,
                                                amount,
-                                               &bc.exchange_auth,
+                                               &cred.ba,
                                                payer_payto);
 }
 
@@ -189,6 +172,7 @@ run (void *cls,
                               "EUR:0",
                               MHD_HTTP_OK),
     TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-1",
+                                             cred.cfg,
                                              merchant_url,
                                              MHD_HTTP_OK,
                                              "1", /* order ID */
@@ -217,7 +201,7 @@ run (void *cls,
                                                  merchant_url,
                                                  "deposit-simple",
                                                  "session-1",
-                                                 MHD_HTTP_NO_CONTENT),
+                                                 MHD_HTTP_OK),
     TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-1"),
     CMD_EXEC_AGGREGATOR ("run-aggregator"),
     /* KYC: hence nothing happened at the bank yet: */
@@ -228,7 +212,8 @@ run (void *cls,
                                         NULL,
                                         NULL,
                                         EXCHANGE_URL,
-                                        MHD_HTTP_NO_CONTENT),
+                                        MHD_HTTP_NO_CONTENT,
+                                        TALER_AML_NORMAL),
     /* now we get the legi UUID */
     TALER_TESTING_cmd_merchant_get_order ("get-order-kyc",
                                           merchant_url,
@@ -243,7 +228,8 @@ run (void *cls,
                                         NULL,
                                         NULL,
                                         EXCHANGE_URL,
-                                        MHD_HTTP_ACCEPTED),
+                                        MHD_HTTP_ACCEPTED,
+                                        TALER_AML_NORMAL),
     TALER_TESTING_cmd_proof_kyc_oauth2 ("kyc-do",
                                         "kyc-pending",
                                         "kyc-provider-test-oauth2",
@@ -256,56 +242,175 @@ run (void *cls,
                                            exchange_payto,
                                            merchant_payto),
     TALER_TESTING_cmd_merchant_post_transfer ("post-transfer-1",
-                                              &bc.exchange_auth,
-                                              PAYTO_I1,
+                                              &cred.ba,
+                                              merchant_payto,
                                               merchant_url,
                                               "EUR:4.98",
-                                              MHD_HTTP_OK,
+                                              MHD_HTTP_NO_CONTENT,
                                               "deposit-simple",
                                               NULL),
+    TALER_TESTING_cmd_run_tme ("run taler-merchant-exchange-1",
+                               CONFIG_FILE),
     TALER_TESTING_cmd_merchant_get_transfers ("get-transfers-1",
                                               merchant_url,
-                                              PAYTO_I1,
+                                              merchant_payto,
                                               MHD_HTTP_OK,
                                               "post-transfer-1",
                                               NULL),
     TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-3"),
     TALER_TESTING_cmd_end ()
   };
+  struct TALER_TESTING_Command aml[] = {
+    TALER_TESTING_cmd_set_officer ("aml-officer",
+                                   NULL,
+                                   "Ernest&Young",
+                                   true,
+                                   false),
+    cmd_transfer_to_exchange ("create-reserve-big",
+                              "EUR:100.02"),
+    TALER_TESTING_cmd_exec_wirewatch ("wirewatch-big",
+                                      CONFIG_FILE),
+    TALER_TESTING_cmd_take_aml_decision ("freeze",
+                                         "aml-officer",
+                                         "post-transfer-1",
+                                         "EUR:1",
+                                         "suspicious",
+                                         TALER_AML_FROZEN,
+                                         NULL,
+                                         MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_bank_admin_transfer ("check_bank_transfer-big",
+                                                 "EUR:100.02",
+                                                 payer_payto,
+                                                 exchange_payto,
+                                                 "create-reserve-big"),
+    TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-aml",
+                                       "create-reserve-big",
+                                       "EUR:5",
+                                       0,
+                                       MHD_HTTP_OK),
+    TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-aml",
+                                             cred.cfg,
+                                             merchant_url,
+                                             MHD_HTTP_OK,
+                                             "10-aml", /* order ID */
+                                             GNUNET_TIME_UNIT_ZERO_TS,
+                                             GNUNET_TIME_UNIT_FOREVER_TS,
+                                             true,
+                                             "EUR:5.0",
+                                             "x-taler-bank",
+                                             "",
+                                             "",
+                                             NULL),
+    TALER_TESTING_cmd_merchant_claim_order ("reclaim-aml",
+                                            merchant_url,
+                                            MHD_HTTP_OK,
+                                            "create-proposal-aml",
+                                            NULL),
+    TALER_TESTING_cmd_merchant_pay_order ("deposit-simple",
+                                          merchant_url,
+                                          MHD_HTTP_OK,
+                                          "create-proposal-aml",
+                                          "withdraw-coin-aml",
+                                          "EUR:5",
+                                          "EUR:4.99",
+                                          "session-aml"),
+    TALER_TESTING_cmd_merchant_post_orders_paid ("verify-order-aml-paid",
+                                                 merchant_url,
+                                                 "deposit-simple",
+                                                 "session-aml",
+                                                 MHD_HTTP_OK),
+    TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-aml-1"),
+    CMD_EXEC_AGGREGATOR ("run-aggregator-aml-frozen"),
+    /* AML-frozen: hence nothing happened at the bank yet: */
+    TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-aml-2"),
+    /* Now we should get a status of frozen */
+    TALER_TESTING_cmd_merchant_kyc_get ("aml-frozen",
+                                        merchant_url,
+                                        NULL, /* no instance ID */
+                                        NULL, /* no wire ref */
+                                        EXCHANGE_URL,
+                                        MHD_HTTP_ACCEPTED,
+                                        TALER_AML_FROZEN),
+    TALER_TESTING_cmd_sleep ("sleep to de-collide AML timestamps",
+                             1),
+    TALER_TESTING_cmd_take_aml_decision ("unfreeze",
+                                         "aml-officer",
+                                         "post-transfer-1",
+                                         "EUR:100",
+                                         "fine",
+                                         TALER_AML_NORMAL,
+                                         NULL,
+                                         MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_kyc_get ("aml-unfrozen",
+                                        merchant_url,
+                                        NULL, /* no instance ID */
+                                        NULL, /* no wire ref */
+                                        EXCHANGE_URL,
+                                        MHD_HTTP_NO_CONTENT,
+                                        TALER_AML_NORMAL),
+    CMD_EXEC_AGGREGATOR ("run-aggregator-aml-normal"),
+    TALER_TESTING_cmd_check_bank_transfer (
+      "check_bank_transfer-498c-post-unfreeze",
+      EXCHANGE_URL,
+      "EUR:4.98",
+      exchange_payto,
+      merchant_payto),
+    TALER_TESTING_cmd_merchant_post_transfer ("post-transfer-aml",
+                                              &cred.ba,
+                                              merchant_payto,
+                                              merchant_url,
+                                              "EUR:4.98",
+                                              MHD_HTTP_NO_CONTENT,
+                                              "deposit-simple",
+                                              NULL),
+    TALER_TESTING_cmd_run_tme ("run taler-merchant-exchange-2-aml",
+                               CONFIG_FILE),
+    TALER_TESTING_cmd_merchant_get_transfers ("get-transfers-aml",
+                                              merchant_url,
+                                              merchant_payto,
+                                              MHD_HTTP_OK,
+                                              "post-transfer-1",
+                                              "post-transfer-aml",
+                                              NULL),
+    TALER_TESTING_cmd_end ()
+  }; /* end of aml batch */
 
   struct TALER_TESTING_Command commands[] = {
     /* general setup */
+    TALER_TESTING_cmd_run_fakebank ("run-fakebank",
+                                    cred.cfg,
+                                    "exchange-account-exchange"),
+    TALER_TESTING_cmd_system_start ("start-taler",
+                                    CONFIG_FILE,
+                                    "-ema",
+                                    "-u", "exchange-account-exchange",
+                                    NULL),
+    TALER_TESTING_cmd_get_exchange ("get-exchange",
+                                    cred.cfg,
+                                    NULL,
+                                    true,
+                                    true),
     TALER_TESTING_cmd_oauth ("start-oauth-service",
                              6666),
-    TALER_TESTING_cmd_auditor_add ("add-auditor-OK",
-                                   MHD_HTTP_NO_CONTENT,
-                                   false),
-    TALER_TESTING_cmd_wire_add ("add-wire-account",
-                                "payto://x-taler-bank/localhost/2?receiver-name=2",
-                                MHD_HTTP_NO_CONTENT,
-                                false),
-    TALER_TESTING_cmd_exec_offline_sign_keys ("offline-sign-future-keys",
-                                              CONFIG_FILE),
-    TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
-                                              CONFIG_FILE,
-                                              "EUR:0.01",
-                                              "EUR:0.01"),
-    TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
-                                                1),
     TALER_TESTING_cmd_merchant_post_instances ("instance-create-default-setup",
                                                merchant_url,
                                                "default",
-                                               PAYTO_I1,
-                                               "EUR",
                                                MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_account (
+        "instance-create-default-account",
+        merchant_url,
+        merchant_payto,
+        NULL, NULL,
+        MHD_HTTP_OK),
     TALER_TESTING_cmd_batch ("pay",
                              pay),
+    TALER_TESTING_cmd_batch ("aml",
+                             aml),
     TALER_TESTING_cmd_end ()
   };
 
-  TALER_TESTING_run_with_fakebank (is,
-                                   commands,
-                                   bc.exchange_auth.wire_gateway_url);
+  TALER_TESTING_run (is,
+                     commands);
 }
 
 
@@ -313,67 +418,27 @@ int
 main (int argc,
       char *const *argv)
 {
-  enum GNUNET_GenericReturnValue ret;
-
-  /* These environment variables get in the way... */
-  unsetenv ("XDG_DATA_HOME");
-  unsetenv ("XDG_CONFIG_HOME");
-  GNUNET_log_setup ("test-kyc-api",
-                    "INFO",
-                    NULL);
-  if (GNUNET_OK !=
-      TALER_TESTING_prepare_fakebank (CONFIG_FILE,
-                                      "exchange-account-exchange",
-                                      &bc))
-    return 77;
-
   payer_payto =
-    ("payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name="
-     USER_ACCOUNT_NAME);
+    "payto://x-taler-bank/localhost/" USER_ACCOUNT_NAME "?receiver-name="
+    USER_ACCOUNT_NAME;
   exchange_payto =
-    ("payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME "?receiver-name="
-     EXCHANGE_ACCOUNT_NAME);
+    "payto://x-taler-bank/localhost/" EXCHANGE_ACCOUNT_NAME "?receiver-name="
+    EXCHANGE_ACCOUNT_NAME;
   merchant_payto =
-    ("payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME "?receiver-name="
-     MERCHANT_ACCOUNT_NAME);
-
-  if (NULL ==
-      (merchant_url = TALER_TESTING_prepare_merchant (CONFIG_FILE)))
-    return 77;
+    "payto://x-taler-bank/localhost/" MERCHANT_ACCOUNT_NAME "?receiver-name="
+    MERCHANT_ACCOUNT_NAME;
+  merchant_url = "http://localhost:8080/";
   GNUNET_asprintf (&merchant_url_i1a,
                    "%sinstances/i1a/",
                    merchant_url);
-  TALER_TESTING_cleanup_files (CONFIG_FILE);
-
-  switch (TALER_TESTING_prepare_exchange (CONFIG_FILE,
-                                          GNUNET_YES,
-                                          &ec))
-  {
-  case GNUNET_SYSERR:
-    GNUNET_break (0);
-    return 1;
-  case GNUNET_NO:
-    return 77;
-  case GNUNET_OK:
-    if (NULL == (merchantd =
-                   TALER_TESTING_run_merchant (CONFIG_FILE,
-                                               merchant_url)))
-      return 1;
-    ret = TALER_TESTING_setup_with_exchange (&run,
-                                             NULL,
-                                             CONFIG_FILE);
-    GNUNET_OS_process_kill (merchantd, SIGTERM);
-    GNUNET_OS_process_wait (merchantd);
-    GNUNET_OS_process_destroy (merchantd);
-    GNUNET_free (merchant_url);
-    if (GNUNET_OK != ret)
-      return 1;
-    break;
-  default:
-    GNUNET_break (0);
-    return 1;
-  }
-  return 0;
+  return TALER_TESTING_main (argv,
+                             "INFO",
+                             CONFIG_FILE,
+                             "exchange-account-exchange",
+                             TALER_TESTING_BS_FAKEBANK,
+                             &cred,
+                             &run,
+                             NULL);
 }
 
 
