@@ -1,6 +1,6 @@
 #!/bin/bash
 # This file is part of TALER
-# Copyright (C) 2014-2021 Taler Systems SA
+# Copyright (C) 2014-2023 Taler Systems SA
 #
 # TALER is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -17,9 +17,11 @@
 # <http://www.gnu.org/licenses/>
 #
 
-. initialize_taler_system.sh
+. setup.sh
+
+# Launch only the merchant.
+setup -c test_template.conf -m
 
-# echo -n "Configuring merchant instance ..."
 
 STATUS=$(curl -H "Content-Type: application/json" -X OPTIONS \
     http://localhost:9966/instances/default/private/products \
@@ -27,8 +29,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X OPTIONS \
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'options should return 204 when default instance doest not exist yet. got:' $STATUS
-    exit 1
+    exit_fail "Expected 204 when default instance does not exist yet. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
@@ -38,20 +39,18 @@ STATUS=$(curl -H "Content-Type: application/json" -X GET \
 
 if [ "$STATUS" != "404" ]
 then
-    echo 'backend should respond 404 when the default instance is not yet created. got:' $STATUS
-    exit 1
+    exit_fail "Expected 404 when the default instance is not yet created. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:super_secret' \
     http://localhost:9966/management/instances \
-    -d '{"auth":{"method":"token","token":"secret-token:other_secret"},"payto_uris":["payto://x-taler-bank/localhost/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_us" : 3600000000},"default_pay_delay":{"d_us": 3600000000}}' \
+    -d '{"auth":{"method":"token","token":"secret-token:other_secret"},"id":"default","name":"default","user_type":"business","address":{},"jurisdiction":{},"use_stefan":true,"default_wire_transfer_delay":{"d_us" : 3600000000},"default_pay_delay":{"d_us": 3600000000}}' \
     -w "%{http_code}" -s -o /dev/null)
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'should respond ok, instance created. got:' $STATUS
-    exit 1
+    exit_fail "Expected 204, instance created. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
@@ -60,8 +59,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X GET \
 
 if [ "$STATUS" != "401" ]
 then
-    echo 'should respond unauthorized without the token for the list of product when the default instance was created. got:' $STATUS
-    exit 1
+    exit_fail "Expected 401 without the token for the list of product when the default instance was created. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
@@ -71,8 +69,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X GET \
 
 if [ "$STATUS" != "200" ]
 then
-    echo 'should respond ok for the list of product when the default instance was created. got:' $STATUS
-    exit 1
+    exit_fail "Expected 200 for the list of product when the default instance was created. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
@@ -83,8 +80,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X POST \
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'should respond ok, instance auth token changed. got:' $STATUS
-    exit 1
+    exit_fail "Expected 204, instance auth token changed. got: $STATUS"
 fi
 
 
@@ -95,8 +91,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
 
 if [ "$STATUS" != "401" ]
 then
-    echo 'should respond unauthorized without the token, when purging the instance. got:' $STATUS
-    exit 1
+    exit_fail "Expected 401 without the token, when purging the instance. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
@@ -106,8 +101,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
 
 if [ "$STATUS" != "401" ]
 then
-    echo 'should respond unauthorized using old token, when purging the instance. got:' $STATUS
-    exit 1
+    exit_fail "Expected 401 using old token, when purging the instance. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
@@ -117,8 +111,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X DELETE \
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'should respond unauthorized without the token, when purging the instance. got:' $STATUS
-    exit 1
+    exit_fail "Expected 204 when purging the instance. got: $STATUS"
 fi
 
 STATUS=$(curl -H "Content-Type: application/json" -X GET \
@@ -128,10 +121,9 @@ STATUS=$(curl -H "Content-Type: application/json" -X GET \
 
 if [ "$STATUS" != "404" ]
 then
-    echo 'should respond not found when trying to list the product and the default instance was deleted. got:' $STATUS
-    exit 1
+    exit_fail "Expected 404 when trying to list the product and the default instance was deleted. got: $STATUS"
 fi
 
-echo "OK"
+echo "Test PASSED"
 
 exit 0
