@@ -133,17 +133,19 @@ delete_reserve_run (void *cls,
 
   drs->is = is;
   if (drs->purge)
-    drs->rdh = TALER_MERCHANT_reserve_purge (is->ctx,
-                                             drs->merchant_url,
-                                             reserve_pub,
-                                             &delete_reserve_cb,
-                                             drs);
+    drs->rdh = TALER_MERCHANT_reserve_purge (
+      TALER_TESTING_interpreter_get_context (is),
+      drs->merchant_url,
+      reserve_pub,
+      &delete_reserve_cb,
+      drs);
   else
-    drs->rdh = TALER_MERCHANT_reserve_delete (is->ctx,
-                                              drs->merchant_url,
-                                              reserve_pub,
-                                              &delete_reserve_cb,
-                                              drs);
+    drs->rdh = TALER_MERCHANT_reserve_delete (
+      TALER_TESTING_interpreter_get_context (is),
+      drs->merchant_url,
+      reserve_pub,
+      &delete_reserve_cb,
+      drs);
 
   GNUNET_assert (NULL != drs->rdh);
 }
