@@ -156,14 +156,14 @@ merchant_get_order_cb (
   switch (osr->hr.http_status)
   {
   case MHD_HTTP_OK:
-    if (gos->osc != osr->details.success.status)
+    if (gos->osc != osr->details.ok.status)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order paid does not match\n");
       TALER_TESTING_interpreter_fail (gos->is);
       return;
     }
-    switch (osr->details.success.status)
+    switch (osr->details.ok.status)
     {
     case TALER_MERCHANT_OSC_PAID:
       {
@@ -212,7 +212,7 @@ merchant_get_order_cb (
 
             for (unsigned int j = 0; j < *paths_length; ++j)
             {
-              const char **path;
+              const char *path;
               int res = GNUNET_OK;
 
               if (GNUNET_OK !=
@@ -228,7 +228,7 @@ merchant_get_order_cb (
 
               GNUNET_assert (GNUNET_OK ==
                              TALER_JSON_expand_path (ct,
-                                                     *path,
+                                                     path,
                                                      &apply_forget,
                                                      &res));
               GNUNET_assert (GNUNET_OK == res);
@@ -236,7 +236,7 @@ merchant_get_order_cb (
           }
 
           if (1 != json_equal (ct,
-                               osr->details.success.details.paid.contract_terms))
+                               osr->details.ok.details.paid.contract_terms))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                         "Order contract terms do not match\n");
@@ -246,14 +246,14 @@ merchant_get_order_cb (
 
           json_decref (ct);
         }
-        if (gos->wired != osr->details.success.details.paid.wired)
+        if (gos->wired != osr->details.ok.details.paid.wired)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order wired does not match\n");
           TALER_TESTING_interpreter_fail (gos->is);
           return;
         }
-        if (gos->transfers_length != osr->details.success.details.paid.wts_len)
+        if (gos->transfers_length != osr->details.ok.details.paid.wts_len)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Number of transfers found does not match\n");
@@ -280,7 +280,7 @@ merchant_get_order_cb (
               return;
             }
             if (0 != GNUNET_memcmp (wtid,
-                                    &osr->details.success.details.paid.wts[i].
+                                    &osr->details.ok.details.paid.wts[i].
                                     wtid))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -290,7 +290,7 @@ merchant_get_order_cb (
             }
           }
           {
-            const char **exchange_url;
+            const char *exchange_url;
 
             if (GNUNET_OK !=
                 TALER_TESTING_get_trait_exchange_url (transfer_cmd,
@@ -301,9 +301,9 @@ merchant_get_order_cb (
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            if (0 != strcmp (*exchange_url,
-                             osr->details.success.details.paid.wts[i].
-                             exchange_url))
+            if (0 != strcmp (
+                  exchange_url,
+                  osr->details.ok.details.paid.wts[i].exchange_url))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Wire transfer exchange url does not match\n");
@@ -311,48 +311,8 @@ merchant_get_order_cb (
               return;
             }
           }
-          {
-            struct TALER_Amount transfer_total;
-            const struct TALER_Amount *transfer_amount;
-            const struct TALER_Amount *transfer_fee;
-
-            if ((GNUNET_OK !=
-                 TALER_TESTING_get_trait_amount (transfer_cmd,
-                                                 &transfer_amount)) ||
-                (GNUNET_OK !=
-                 TALER_TESTING_get_trait_fee (transfer_cmd,
-                                              &transfer_fee)))
-            {
-              GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                          "Could not fetch wire transfer amount/fee\n");
-              TALER_TESTING_interpreter_fail (gos->is);
-              return;
-            }
-            if (0 > TALER_amount_add (&transfer_total,
-                                      transfer_amount,
-                                      transfer_fee))
-            {
-              GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                          "Could not total wire transfer\n");
-              TALER_TESTING_interpreter_fail (gos->is);
-              return;
-            }
-            if ((GNUNET_OK !=
-                 TALER_amount_cmp_currency (
-                   &transfer_total,
-                   &osr->details.success.details.paid.wts[i].total_amount)) ||
-                (0 != TALER_amount_cmp (
-                   &transfer_total,
-                   &osr->details.success.details.paid.wts[i].total_amount)))
-            {
-              GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                          "Wire transfer total does not match\n");
-              TALER_TESTING_interpreter_fail (gos->is);
-              return;
-            }
-          }
         }
-        if (gos->refunded != osr->details.success.details.paid.refunded)
+        if (gos->refunded != osr->details.ok.details.paid.refunded)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order refunded does not match\n");
@@ -360,7 +320,7 @@ merchant_get_order_cb (
           return;
         }
         if (gos->refunds_length !=
-            osr->details.success.details.paid.refunds_len)
+            osr->details.ok.details.paid.refunds_len)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Number of refunds found does not match\n");
@@ -371,7 +331,7 @@ merchant_get_order_cb (
           GNUNET_assert (
             GNUNET_OK ==
             TALER_amount_set_zero (
-              osr->details.success.details.paid.refund_amount.currency,
+              osr->details.ok.details.paid.refund_amount.currency,
               &refunded_total));
         for (unsigned int i = 0; i < gos->refunds_length; ++i)
         {
@@ -383,7 +343,7 @@ merchant_get_order_cb (
           {
             const struct TALER_Amount *expected_amount;
             struct TALER_Amount *amount_found =
-              &osr->details.success.details.paid.refunds[i].refund_amount;
+              &osr->details.ok.details.paid.refunds[i].refund_amount;
 
             if (GNUNET_OK !=
                 TALER_TESTING_get_trait_amount (refund_cmd,
@@ -410,7 +370,7 @@ merchant_get_order_cb (
             }
           }
           {
-            const char **expected_reason;
+            const char *expected_reason;
 
             if (GNUNET_OK !=
                 TALER_TESTING_get_trait_reason (refund_cmd,
@@ -421,9 +381,10 @@ merchant_get_order_cb (
               TALER_TESTING_interpreter_fail (gos->is);
               return;
             }
-            if (0 != strcmp (
-                  *expected_reason,
-                  osr->details.success.details.paid.refunds[i].reason))
+            if (0 !=
+                strcmp (
+                  expected_reason,
+                  osr->details.ok.details.paid.refunds[i].reason))
             {
               GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                           "Refund reason does not match\n");
@@ -433,7 +394,7 @@ merchant_get_order_cb (
           }
         }
 
-        if (gos->wired != osr->details.success.details.paid.wired)
+        if (gos->wired != osr->details.ok.details.paid.wired)
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order wired does not match\n");
@@ -449,17 +410,17 @@ merchant_get_order_cb (
       {
         struct TALER_MERCHANT_PayUriData pud;
         const struct TALER_TESTING_Command *order_cmd;
-        const char **order_id;
+        const char *order_id;
         const struct TALER_ClaimTokenP *claim_token;
 
         if (GNUNET_OK !=
             TALER_MERCHANT_parse_pay_uri (
-              osr->details.success.details.unpaid.taler_pay_uri,
+              osr->details.ok.details.unpaid.taler_pay_uri,
               &pud))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Taler pay uri `%s' is malformed\n",
-                      osr->details.success.details.unpaid.taler_pay_uri);
+                      osr->details.ok.details.unpaid.taler_pay_uri);
           TALER_TESTING_interpreter_fail (gos->is);
           return;
         }
@@ -484,37 +445,21 @@ merchant_get_order_cb (
           TALER_TESTING_FAIL (gos->is);
         }
         {
-          char *port;
           char *host;
 
-          if (GNUNET_OK !=
-              GNUNET_CONFIGURATION_get_value_string (gos->is->cfg,
-                                                     "merchant",
-                                                     "PORT",
-                                                     &port))
-          {
-            /* How did we get here without a configured port? */
-            GNUNET_break (0);
-            TALER_TESTING_interpreter_fail (gos->is);
-            TALER_MERCHANT_parse_pay_uri_free (&pud);
-            return;
-          }
-          GNUNET_asprintf (&host,
-                           "localhost:%s",
-                           port);
-          GNUNET_free (port);
+          host = TALER_MERCHANT_TESTING_extract_host (gos->merchant_url);
           if ((0 != strcmp (host,
                             pud.merchant_host)) ||
               (NULL != pud.merchant_prefix_path) ||
-              (0 != strcmp (*order_id,
+              (0 != strcmp (order_id,
                             pud.order_id)) ||
               (NULL != pud.ssid))
           {
             GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                         "Order pay uri `%s' does not match, wanted %s/%s\n",
-                        osr->details.success.details.unpaid.taler_pay_uri,
+                        osr->details.ok.details.unpaid.taler_pay_uri,
                         host,
-                        *order_id);
+                        order_id);
             TALER_TESTING_interpreter_fail (gos->is);
             TALER_MERCHANT_parse_pay_uri_free (&pud);
             GNUNET_free (host);
@@ -568,7 +513,7 @@ merchant_get_order_run (void *cls,
 {
   struct MerchantGetOrderState *gos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char **order_id;
+  const char *order_id;
   const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
@@ -586,14 +531,15 @@ merchant_get_order_run (void *cls,
     TALER_TESTING_FAIL (is);
 
   gos->is = is;
-  gos->ogh = TALER_MERCHANT_merchant_order_get (is->ctx,
-                                                gos->merchant_url,
-                                                *order_id,
-                                                NULL,
-                                                true,
-                                                GNUNET_TIME_UNIT_ZERO,
-                                                &merchant_get_order_cb,
-                                                gos);
+  gos->ogh = TALER_MERCHANT_merchant_order_get (
+    TALER_TESTING_interpreter_get_context (is),
+    gos->merchant_url,
+    order_id,
+    NULL,
+    true,
+    GNUNET_TIME_UNIT_ZERO,
+    &merchant_get_order_cb,
+    gos);
 }
 
 
@@ -612,7 +558,7 @@ merchant_get_order_cleanup (void *cls,
 
   if (NULL != gos->ogh)
   {
-    TALER_LOG_WARNING ("Get tip operation did not complete\n");
+    TALER_LOG_WARNING ("Get order operation did not complete\n");
     TALER_MERCHANT_merchant_order_get_cancel (gos->ogh);
   }
   GNUNET_array_grow (gos->transfers,
@@ -907,14 +853,15 @@ merchant_poll_order_start_run (void *cls,
     = GNUNET_TIME_absolute_add (GNUNET_TIME_relative_to_absolute (pos->timeout),
                                 GNUNET_TIME_UNIT_SECONDS);
   pos->is = is;
-  pos->ogh = TALER_MERCHANT_merchant_order_get (is->ctx,
-                                                pos->merchant_url,
-                                                pos->order_id,
-                                                NULL,
-                                                false,
-                                                pos->timeout,
-                                                &merchant_poll_order_cb,
-                                                pos);
+  pos->ogh = TALER_MERCHANT_merchant_order_get (
+    TALER_TESTING_interpreter_get_context (is),
+    pos->merchant_url,
+    pos->order_id,
+    NULL,
+    false,
+    pos->timeout,
+    &merchant_poll_order_cb,
+    pos);
   GNUNET_assert (NULL != pos->ogh);
   /* We CONTINUE to run the interpreter while the long-polled command
      completes asynchronously! */
