@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2021 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -45,6 +45,11 @@ struct OrdersState
    */
   const char *order_id;
 
+  /**
+   * Our configuration.
+   */
+  const struct GNUNET_CONFIGURATION_Handle *cfg;
+
   /**
    * The order id we expect the merchant to assign (if not NULL).
    */
@@ -157,7 +162,7 @@ orders_traits (void *cls,
 {
   struct OrdersState *ps = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_order_id (&ps->order_id),
+    TALER_TESTING_make_trait_order_id (ps->order_id),
     TALER_TESTING_make_trait_contract_terms (ps->contract_terms),
     TALER_TESTING_make_trait_order_terms (ps->order_terms),
     TALER_TESTING_make_trait_h_contract_terms (&ps->h_contract_terms),
@@ -181,43 +186,41 @@ orders_traits (void *cls,
  * created.
  *
  * @param cls closure
- * @param hr HTTP response we got
- * @param contract_terms contract terms of this order
- * @param sig merchant's signature
- * @param hash hash over the contract
+ * @param ocr response we got
  */
 static void
 orders_claim_cb (void *cls,
-                 const struct TALER_MERCHANT_HttpResponse *hr,
-                 const json_t *contract_terms,
-                 const struct TALER_MerchantSignatureP *sig,
-                 const struct TALER_PrivateContractHashP *hash)
+                 const struct TALER_MERCHANT_OrderClaimResponse *ocr)
 {
   struct OrdersState *ps = cls;
-  struct TALER_MerchantPublicKeyP merchant_pub;
   const char *error_name;
   unsigned int error_line;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("merchant_pub",
-                                 &merchant_pub),
+                                 &ps->merchant_pub),
     GNUNET_JSON_spec_end ()
   };
 
   ps->och = NULL;
-  if (ps->http_status != hr->http_status)
+  if (ps->http_status != ocr->hr.http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Expected status %u, got %u\n",
                 ps->http_status,
-                hr->http_status);
+                ocr->hr.http_status);
     TALER_TESTING_FAIL (ps->is);
   }
-
-  ps->contract_terms = json_deep_copy (contract_terms);
-  ps->h_contract_terms = *hash;
-  ps->merchant_sig = *sig;
+  if (MHD_HTTP_OK != ocr->hr.http_status)
+  {
+    TALER_TESTING_interpreter_next (ps->is);
+    return;
+  }
+  ps->contract_terms = json_deep_copy (
+    (json_t *) ocr->details.ok.contract_terms);
+  ps->h_contract_terms = ocr->details.ok.h_contract_terms;
+  ps->merchant_sig = ocr->details.ok.sig;
   if (GNUNET_OK !=
-      GNUNET_JSON_parse (contract_terms,
+      GNUNET_JSON_parse (ps->contract_terms,
                          spec,
                          &error_name,
                          &error_line))
@@ -236,16 +239,14 @@ orders_claim_cb (void *cls,
     free (log);
     TALER_TESTING_FAIL (ps->is);
   }
-  ps->merchant_pub = merchant_pub;
   TALER_TESTING_interpreter_next (ps->is);
 }
 
 
 /**
- * Callback that processes the response following a
- * POST /orders.  NOTE: no contract terms are included
- * here; they need to be taken via the "orders lookup"
- * method.
+ * Callback that processes the response following a POST /orders.  NOTE: no
+ * contract terms are included here; they need to be taken via the "orders
+ * lookup" method.
  *
  * @param cls closure.
  * @param por details about the response
@@ -259,20 +260,17 @@ order_cb (void *cls,
   ps->po = NULL;
   if (ps->http_status != por->hr.http_status)
   {
-    TALER_LOG_ERROR ("Given vs expected: %u(%d) vs %u\n",
-                     por->hr.http_status,
-                     (int) por->hr.ec,
-                     ps->http_status);
-    TALER_TESTING_FAIL (ps->is);
-  }
-  if (0 == ps->http_status)
-  {
-    TALER_LOG_DEBUG ("/orders, expected 0 status code\n");
-    TALER_TESTING_interpreter_next (ps->is);
-    return;
+    TALER_TESTING_unexpected_status_with_body (ps->is,
+                                               por->hr.http_status,
+                                               ps->http_status,
+                                               por->hr.reply);
   }
   switch (por->hr.http_status)
   {
+  case 0:
+    TALER_LOG_DEBUG ("/orders, expected 0 status code\n");
+    TALER_TESTING_interpreter_next (ps->is);
+    return;
   case MHD_HTTP_OK:
     if (NULL != por->details.ok.token)
       ps->claim_token = *por->details.ok.token;
@@ -351,13 +349,14 @@ order_cb (void *cls,
     return;
   }
   if (NULL ==
-      (ps->och = TALER_MERCHANT_order_claim (ps->is->ctx,
-                                             ps->merchant_url,
-                                             ps->order_id,
-                                             &ps->nonce,
-                                             &ps->claim_token,
-                                             &orders_claim_cb,
-                                             ps)))
+      (ps->och = TALER_MERCHANT_order_claim (
+         TALER_TESTING_interpreter_get_context (ps->is),
+         ps->merchant_url,
+         ps->order_id,
+         &ps->nonce,
+         &ps->claim_token,
+         &orders_claim_cb,
+         ps)))
     TALER_TESTING_FAIL (ps->is);
 }
 
@@ -383,7 +382,7 @@ orders_run (void *cls,
     struct GNUNET_TIME_Absolute now;
     char *order_id;
 
-    now = GNUNET_TIME_absolute_get_monotonic (is->cfg);
+    now = GNUNET_TIME_absolute_get_monotonic (ps->cfg);
     order_id = GNUNET_STRINGS_data_to_string_alloc (
       &now,
       sizeof (now));
@@ -396,7 +395,8 @@ orders_run (void *cls,
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                               &ps->nonce,
                               sizeof (struct GNUNET_CRYPTO_EddsaPublicKey));
-  ps->po = TALER_MERCHANT_orders_post (is->ctx,
+  ps->po = TALER_MERCHANT_orders_post (TALER_TESTING_interpreter_get_context (
+                                         is),
                                        ps->merchant_url,
                                        ps->order_terms,
                                        GNUNET_TIME_UNIT_ZERO,
@@ -456,7 +456,7 @@ orders_run2 (void *cls,
       struct GNUNET_TIME_Absolute now;
       char *order_id;
 
-      now = GNUNET_TIME_absolute_get_monotonic (is->cfg);
+      now = GNUNET_TIME_absolute_get_monotonic (ps->cfg);
       order_id = GNUNET_STRINGS_data_to_string_alloc (
         &now.abs_value_us,
         sizeof (now.abs_value_us));
@@ -514,7 +514,7 @@ orders_run2 (void *cls,
        token = strtok (NULL, ";"))
   {
     const struct TALER_TESTING_Command *lock_cmd;
-    const char **uuid;
+    const char *uuid;
 
     lock_cmd = TALER_TESTING_interpreter_lookup_command (
       is,
@@ -532,9 +532,10 @@ orders_run2 (void *cls,
 
     GNUNET_array_append (locks,
                          locks_length,
-                         *uuid);
+                         uuid);
   }
-  ps->po = TALER_MERCHANT_orders_post2 (is->ctx,
+  ps->po = TALER_MERCHANT_orders_post2 (TALER_TESTING_interpreter_get_context (
+                                          is),
                                         ps->merchant_url,
                                         order,
                                         GNUNET_TIME_UNIT_ZERO,
@@ -702,6 +703,7 @@ TALER_TESTING_cmd_merchant_post_orders_no_claim (
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_merchant_post_orders (
   const char *label,
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
@@ -712,6 +714,7 @@ TALER_TESTING_cmd_merchant_post_orders (
   struct OrdersState *ps;
 
   ps = GNUNET_new (struct OrdersState);
+  ps->cfg = cfg;
   make_order_json (order_id,
                    refund_deadline,
                    pay_deadline,
@@ -738,6 +741,7 @@ TALER_TESTING_cmd_merchant_post_orders (
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_merchant_post_orders2 (
   const char *label,
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
   const char *merchant_url,
   unsigned int http_status,
   const char *order_id,
@@ -753,6 +757,7 @@ TALER_TESTING_cmd_merchant_post_orders2 (
   struct OrdersState *ps;
 
   ps = GNUNET_new (struct OrdersState);
+  ps->cfg = cfg;
   make_order_json (order_id,
                    refund_deadline,
                    pay_deadline,
