@@ -60,9 +60,9 @@ struct PostTemplatesState
   const char *template_description;
 
   /**
-   * base64-encoded key
+   * OTP device ID.
    */
-  char *pos_key;
+  char *otp_id;
 
   /**
    * Contract of the company
@@ -110,6 +110,8 @@ post_templates_cb (void *cls,
     break;
   case MHD_HTTP_NOT_FOUND:
     break;
+  case MHD_HTTP_CONFLICT:
+    break;
   default:
     GNUNET_break (0);
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -136,15 +138,15 @@ post_templates_run (void *cls,
   struct PostTemplatesState *tis = cls;
 
   tis->is = is;
-  tis->iph = TALER_MERCHANT_templates_post (is->ctx,
-                                            tis->merchant_url,
-                                            tis->template_id,
-                                            tis->template_description,
-                                            tis->pos_key,
-                                            TALER_MCA_NONE,
-                                            tis->template_contract,
-                                            &post_templates_cb,
-                                            tis);
+  tis->iph = TALER_MERCHANT_templates_post (
+    TALER_TESTING_interpreter_get_context (is),
+    tis->merchant_url,
+    tis->template_id,
+    tis->template_description,
+    tis->otp_id,
+    tis->template_contract,
+    &post_templates_cb,
+    tis);
   if (NULL == tis->iph)
   {
     GNUNET_break (0);
@@ -172,11 +174,10 @@ post_templates_traits (void *cls,
 {
   struct PostTemplatesState *pts = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_template_description (&pts->template_description),
-    TALER_TESTING_make_trait_template_pos_key (
-      (const char **) &pts->pos_key),
+    TALER_TESTING_make_trait_template_description (pts->template_description),
+    TALER_TESTING_make_trait_otp_id (pts->otp_id),
     TALER_TESTING_make_trait_template_contract (pts->template_contract),
-    TALER_TESTING_make_trait_template_id (&pts->template_id),
+    TALER_TESTING_make_trait_template_id (pts->template_id),
     TALER_TESTING_trait_end (),
   };
 
@@ -206,7 +207,7 @@ post_templates_cleanup (void *cls,
                 "POST /templates operation did not complete\n");
     TALER_MERCHANT_templates_post_cancel (tis->iph);
   }
-  GNUNET_free (tis->pos_key);
+  GNUNET_free (tis->otp_id);
   json_decref (tis->template_contract);
   GNUNET_free (tis);
 }
@@ -218,7 +219,7 @@ TALER_TESTING_cmd_merchant_post_templates2 (
   const char *merchant_url,
   const char *template_id,
   const char *template_description,
-  const char *pos_key,
+  const char *otp_id,
   json_t *template_contract,
   unsigned int http_status)
 {
@@ -232,7 +233,7 @@ TALER_TESTING_cmd_merchant_post_templates2 (
   tis->template_id = template_id;
   tis->http_status = http_status;
   tis->template_description = template_description;
-  tis->pos_key = (NULL == pos_key) ? NULL : GNUNET_strdup (pos_key);
+  tis->otp_id = (NULL == otp_id) ? NULL : GNUNET_strdup (otp_id);
   tis->template_contract = template_contract;
   {
     struct TALER_TESTING_Command cmd = {
