@@ -103,14 +103,14 @@ wallet_get_order_cb (
   switch (hr->http_status)
   {
   case MHD_HTTP_OK:
-    if (gos->refunded != owgr->details.success.refunded)
+    if (gos->refunded != owgr->details.ok.refunded)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order refunded does not match\n");
       TALER_TESTING_interpreter_fail (gos->is);
       return;
     }
-    if (gos->refund_pending != owgr->details.success.refund_pending)
+    if (gos->refund_pending != owgr->details.ok.refund_pending)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order refund pending does not match\n");
@@ -122,7 +122,7 @@ wallet_get_order_cb (
     {
       struct TALER_MERCHANT_PayUriData pud;
       const struct TALER_TESTING_Command *order_cmd;
-      const char **order_id;
+      const char *order_id;
       const struct TALER_ClaimTokenP *claim_token;
 
       if (GNUNET_OK !=
@@ -158,29 +158,13 @@ wallet_get_order_cb (
       }
 
       {
-        char *port;
         char *host;
 
-        if (GNUNET_OK !=
-            GNUNET_CONFIGURATION_get_value_string (gos->is->cfg,
-                                                   "merchant",
-                                                   "PORT",
-                                                   &port))
-        {
-          /* How did we get here without a configured port? */
-          GNUNET_break (0);
-          TALER_TESTING_interpreter_fail (gos->is);
-          TALER_MERCHANT_parse_pay_uri_free (&pud);
-          return;
-        }
-        GNUNET_asprintf (&host,
-                         "localhost:%s",
-                         port);
-        GNUNET_free (port);
+        host = TALER_MERCHANT_TESTING_extract_host (gos->merchant_url);
         if ((0 != strcmp (host,
                           pud.merchant_host)) ||
             (NULL != pud.merchant_prefix_path) ||
-            (0 != strcmp (*order_id,
+            (0 != strcmp (order_id,
                           pud.order_id)) ||
             (NULL != pud.ssid))
         {
@@ -235,7 +219,7 @@ wallet_get_order_run (void *cls,
 {
   struct WalletGetOrderState *gos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char **order_id;
+  const char *order_id;
   const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
@@ -253,16 +237,17 @@ wallet_get_order_run (void *cls,
     TALER_TESTING_FAIL (is);
 
   gos->is = is;
-  gos->ogh = TALER_MERCHANT_wallet_order_get (is->ctx,
-                                              gos->merchant_url,
-                                              *order_id,
-                                              h_contract,
-                                              GNUNET_TIME_UNIT_ZERO,
-                                              NULL,
-                                              NULL,
-                                              false,
-                                              &wallet_get_order_cb,
-                                              gos);
+  gos->ogh = TALER_MERCHANT_wallet_order_get (
+    TALER_TESTING_interpreter_get_context (is),
+    gos->merchant_url,
+    order_id,
+    h_contract,
+    GNUNET_TIME_UNIT_ZERO,
+    NULL,
+    NULL,
+    false,
+    &wallet_get_order_cb,
+    gos);
 }
 
 
@@ -281,7 +266,7 @@ wallet_get_order_cleanup (void *cls,
 
   if (NULL != gos->ogh)
   {
-    TALER_LOG_WARNING ("Get tip operation did not complete\n");
+    TALER_LOG_WARNING ("Get order operation did not complete\n");
     TALER_MERCHANT_wallet_order_get_cancel (gos->ogh);
   }
   GNUNET_free (gos);
@@ -554,10 +539,10 @@ wallet_poll_order_cb (
   {
   case MHD_HTTP_OK:
     pos->paid = true;
-    pos->refunded = owgr->details.success.refunded;
-    pos->refund_pending = owgr->details.success.refund_pending;
-    if (owgr->details.success.refunded)
-      pos->refund_available = owgr->details.success.refund_amount;
+    pos->refunded = owgr->details.ok.refunded;
+    pos->refund_pending = owgr->details.ok.refund_pending;
+    if (owgr->details.ok.refunded)
+      pos->refund_available = owgr->details.ok.refund_amount;
     break;
   case MHD_HTTP_PAYMENT_REQUIRED:
     if (NULL != owgr->details.payment_required.already_paid_order_id)
@@ -593,7 +578,7 @@ wallet_poll_order_start_run (void *cls,
 {
   struct WalletPollOrderStartState *pos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char **order_id;
+  const char *order_id;
   const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
@@ -615,18 +600,19 @@ wallet_poll_order_start_run (void *cls,
     = GNUNET_TIME_absolute_add (GNUNET_TIME_relative_to_absolute (pos->timeout),
                                 GNUNET_TIME_UNIT_SECONDS);
   pos->is = is;
-  pos->ogh = TALER_MERCHANT_wallet_order_get (is->ctx,
-                                              pos->merchant_url,
-                                              *order_id,
-                                              h_contract,
-                                              pos->timeout,
-                                              pos->session_id,
-                                              pos->wait_for_refund
-                                              ? &pos->refund_threshold
-                                              : NULL,
-                                              false, /* await_refund_obtained */
-                                              &wallet_poll_order_cb,
-                                              pos);
+  pos->ogh = TALER_MERCHANT_wallet_order_get (
+    TALER_TESTING_interpreter_get_context (is),
+    pos->merchant_url,
+    order_id,
+    h_contract,
+    pos->timeout,
+    pos->session_id,
+    pos->wait_for_refund
+    ? &pos->refund_threshold
+    : NULL,
+    false,                                           /* await_refund_obtained */
+    &wallet_poll_order_cb,
+    pos);
   GNUNET_assert (NULL != pos->ogh);
   /* We CONTINUE to run the interpreter while the long-polled command
      completes asynchronously! */
